"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NavigatorView = void 0;

var _react = _interopRequireDefault(require("react"));

var _reactNative = require("react-native");

var _RNIWrapperView = require("../native_components/RNIWrapperView");

var _RNINavigatorView = require("../native_components/RNINavigatorView");

var _RNINavigatorViewModule = require("../native_modules/RNINavigatorViewModule");

var _NavigatorRouteView = require("./NavigatorRouteView");

var _NavigatorUIConstantsContext = require("../context/NavigatorUIConstantsContext");

var Helpers = _interopRequireWildcard(require("../functions/Helpers"));

var _EventEmitter = require("../functions/EventEmitter");

var _SimpleQueue = require("../functions/SimpleQueue");

var _LibraryConstants = require("../constants/LibraryConstants");

function _getRequireWildcardCache() { if (typeof WeakMap !== "function") return null; var cache = new WeakMap(); _getRequireWildcardCache = function () { return cache; }; return cache; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/** Represents the current status of the navigator */
var NavStatus;

(function (NavStatus) {
  NavStatus["IDLE"] = "IDLE";
  NavStatus["IDLE_INIT"] = "IDLE_INIT";
  NavStatus["IDLE_ERROR"] = "IDLE_ERROR";
  NavStatus["NAV_PUSHING"] = "NAV_PUSHING";
  NavStatus["NAV_POPPING"] = "NAV_POPPING";
  NavStatus["NAV_REMOVING"] = "NAV_REMOVING";
  NavStatus["NAV_REPLACING"] = "NAV_REPLACING";
  NavStatus["NAV_INSERTING"] = "NAV_INSERTING";
  NavStatus["NAV_UPDATING"] = "NAV_UPDATING";
  NavStatus["UNMOUNTED"] = "UNMOUNTED";
  NavStatus["NAV_ABORT"] = "NAV_ABORT";
})(NavStatus || (NavStatus = {}));

;
var NavEvents;

(function (NavEvents) {
  NavEvents["onNavRouteViewAdded"] = "onNavRouteViewAdded";
  NavEvents["onSetNativeRoutes"] = "onSetNativeRoutes";
})(NavEvents || (NavEvents = {}));

;
/** Represents a route in the nav. `state.activeRoutes` */

//#endregion
const TIMEOUT_MOUNT = 750;
const TIMEOUT_COMMAND = 1500;
let NAVIGATOR_ID_COUNTER = 0;
let ROUTE_ID_COUNTER = 0;
let nativeRouteKeys = null;

class NavigatorView extends _react.default.PureComponent {
  //#region - Property Declarations

  /** A ref to the `RNINavigatorView` native component. */

  /** Unique identifier for this navigator */

  /** Used for `removeRouteBatchedFromState`*/

  /** Queue for nav. commands: only 1 command is called at a time */
  // note: the key should be the routeID
  //#endregion
  constructor(props) {
    super(props);

    _defineProperty(this, "state", void 0);

    _defineProperty(this, "nativeRef", void 0);

    _defineProperty(this, "navigatorID", void 0);

    _defineProperty(this, "navStatus", void 0);

    _defineProperty(this, "emitter", void 0);

    _defineProperty(this, "routesToRemove", void 0);

    _defineProperty(this, "queue", void 0);

    _defineProperty(this, "routeRefMap", {});

    _defineProperty(this, "isValidRouteKey", routeKey => {
      var _nativeRouteKeys;

      const {
        routes
      } = this.props;
      const hasNativeRouteKeyMatch = ((_nativeRouteKeys = nativeRouteKeys) === null || _nativeRouteKeys === void 0 ? void 0 : _nativeRouteKeys[routeKey]) != null;
      return hasNativeRouteKeyMatch || routes.some(route => route.routeKey === routeKey);
    });

    _defineProperty(this, "getInitialRoutes", () => {
      const {
        initialRoutes
      } = this.props; // guard: initial validation for `initialRoutes`

      const isValid = initialRoutes != null && Array.isArray(initialRoutes);
      const isEmpty = (initialRoutes === null || initialRoutes === void 0 ? void 0 : initialRoutes.length) === 0;

      if (!isValid || isEmpty) {
        throw new Error("'NavigatorView' error: Invalid value given to 'initialRoutes' prop.");
      }

      ;
      return initialRoutes.map((route, index) => {
        const config = this.getRouteConfig(route.routeKey); // guard: no matching route config found for `routeKey`

        if (config == null) {
          throw new Error("'NavigatorView' error: invalid value for 'initialRoutes' prop" + " - no matching route found for 'routeKey': ".concat(route.routeKey, ".") + " All routes must be declared in the routes prop." + " If this is a native route, please add it to the 'routes' prop w/" + " the 'isNativeRoute' property set to true.");
        }

        ;
        const routeItem = {
          isNativeRoute: config.isNativeRoute,
          routeID: ROUTE_ID_COUNTER++,
          routeIndex: index,
          routeKey: route.routeKey,
          routeProps: route.routeProps
        };
        return config.isNativeRoute ? routeItem : { ...routeItem,
          routeOptions: config.routeOptionsDefault
        };
      });
    });

    _defineProperty(this, "getNativeRoutes", () => {
      const {
        activeRoutes
      } = this.state;
      const nativeRoutes = activeRoutes.filter(route => route.isNativeRoute);
      return nativeRoutes.reduce((acc, curr) => {
        acc[curr.routeID] = {
          routeKey: curr.routeKey,
          routeIndex: curr.routeIndex,
          routeProps: curr.routeProps
        };
        return acc;
      }, {});
    });

    _defineProperty(this, "getRoutesToRender", () => {
      // make a copy
      const activeRoutes = [...this.state.activeRoutes];
      return activeRoutes // only return js/react routes
      .filter(route => !route.isNativeRoute) // sort by routeID in asc. order
      .sort((a, b) => a.routeID - b.routeID);
    });

    _defineProperty(this, "getRouteConfig", routeKey => {
      var _nativeRouteKeys2;

      const {
        routes
      } = this.props;
      const routeConfig = routes.find(item => item.routeKey === routeKey);
      const nativeRouteKey = (_nativeRouteKeys2 = nativeRouteKeys) === null || _nativeRouteKeys2 === void 0 ? void 0 : _nativeRouteKeys2[routeKey];
      const hasNativeRouteKeyMatch = nativeRouteKey != null;
      const isNativeRoute = (routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) || hasNativeRouteKeyMatch;
      return !isNativeRoute ? routeConfig : {
        routeKey,
        isNativeRoute: true,
        initialRouteProps: routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.initialRouteProps
      };
    });

    _defineProperty(this, "getLastRouteTransitionDuration", isPushing => {
      var _routeConfig$transiti, _routeConfig$transiti2, _routeConfig$transiti3, _routeConfig$transiti4;

      const {
        activeRoutes
      } = this.state;
      const lastIndex = activeRoutes.length - 1;
      const lastRoute = activeRoutes[lastIndex];
      const lastRouteRef = this.routeRefMap[lastRoute.routeID]; // if the last route is a native route, then this will be null

      const routeConfig = lastRouteRef === null || lastRouteRef === void 0 ? void 0 : lastRouteRef.getRouteOptions();
      return isPushing ? (_routeConfig$transiti = routeConfig === null || routeConfig === void 0 ? void 0 : (_routeConfig$transiti2 = routeConfig.transitionConfigPush) === null || _routeConfig$transiti2 === void 0 ? void 0 : _routeConfig$transiti2.duration) !== null && _routeConfig$transiti !== void 0 ? _routeConfig$transiti : 0 : (_routeConfig$transiti3 = routeConfig === null || routeConfig === void 0 ? void 0 : (_routeConfig$transiti4 = routeConfig.transitionConfigPop) === null || _routeConfig$transiti4 === void 0 ? void 0 : _routeConfig$transiti4.duration) !== null && _routeConfig$transiti3 !== void 0 ? _routeConfig$transiti3 : 0;
    });

    _defineProperty(this, "configureTransitionOverride", params => {
      var _params$pushConfig, _params$popConfig, _ref;

      const hasTransitionConfig = params.pushConfig != null || params.popConfig != null; // the push/pop duration in seconds

      const transitionConfigDuration = params.isPushing ? (_params$pushConfig = params.pushConfig) === null || _params$pushConfig === void 0 ? void 0 : _params$pushConfig.duration : (_params$popConfig = params.popConfig) === null || _params$popConfig === void 0 ? void 0 : _params$popConfig.duration; // note: convert seconds -> ms

      const transitionDuration = 1000 * ((_ref = transitionConfigDuration !== null && transitionConfigDuration !== void 0 ? transitionConfigDuration : this.getLastRouteTransitionDuration(true)) !== null && _ref !== void 0 ? _ref : 0);

      if (transitionDuration > 10000) {
        throw new Error("The transition duration of ".concat(transitionDuration, " sec. is too long") + " - reminder: specify duration in seconds (ex: 0.5), not in ms (ex: 500)");
      }

      ;
      return {
        // add some wiggle room
        transitionDuration: transitionDuration + 250,
        setTransition: async () => {
          if (hasTransitionConfig) {
            await Promise.all([// temp bugfix: delay so that pop transition config is set/received
            // from native.
            !params.isPushing && Helpers.timeout(100), // temporarily override the last route's push/pop transition
            Helpers.setStateAsync(this, {
              transitionConfigPushOverride: params.pushConfig,
              transitionConfigPopOverride: params.popConfig
            })]);
          }

          ;
        },
        resetTransition: async () => {
          if (hasTransitionConfig) {
            // temporarily override the last route's push/pop transition
            await Helpers.setStateAsync(this, {
              transitionConfigPushOverride: null,
              transitionConfigPopOverride: null
            });
          }

          ;
        }
      };
    });

    _defineProperty(this, "waitForRoutes", routeItems => {
      // filter out/separate react and native routes
      const [nativeRoutes, reactRoutes] = routeItems.reduce((acc, curr) => {
        acc[curr.isNativeRoute ? 0 : 1].push(curr.routeID);
        return acc;
      }, [[], []]);
      const hasNativeRoutes = nativeRoutes.length > 0;
      return Helpers.promiseWithTimeout(TIMEOUT_MOUNT, Promise.all([// 1. wait for native routes to be init.
      hasNativeRoutes && new Promise(resolve => {
        this.emitter.once(NavEvents.onSetNativeRoutes, () => {
          resolve();
        });
      }), // 2. wait for react routes to be "received" from native
      ...reactRoutes.map(routeID => new Promise(resolve => {
        this.emitter.once(NavEvents.onNavRouteViewAdded, ({
          nativeEvent
        }) => {
          if (nativeEvent.routeID === routeID) {
            resolve();
          }

          ;
        });
      }))]));
    });

    _defineProperty(this, "createStateSnapshot", () => {
      let stateSnapshot = null;
      return {
        save: () => {
          stateSnapshot = { ...this.state
          };
        },
        restore: () => {
          const isMounted = this.navStatus !== NavStatus.UNMOUNTED;

          if (stateSnapshot != null && isMounted) {
            this.setState(stateSnapshot);
            stateSnapshot = null;
          }

          ;
        },
        clear: () => {
          stateSnapshot = null;
        }
      };
    });

    _defineProperty(this, "removeRouteBatchedFromState", async params => {
      if (params) {
        // To prevent too many state updates, the routes to be removed
        // are queued/grouped, and are removed in batches...
        this.routesToRemove.push(params);
      }

      ; // trigger remove if:

      const shouldRemove = () => // - A the route to be removed has a valid `routeKey`
      this.isValidRouteKey(params.routeKey) && // - B. there are queued items to remove
      this.routesToRemove.length > 0; // if busy, wait for prev. to finish


      const queue = this.queue.schedule();
      await queue.promise;
      this.setNavStatus(NavStatus.NAV_POPPING);

      while (shouldRemove()) {
        //#region - 🐞 DEBUG 🐛
        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, removeRouteBatchedFromState" + " - with routeKey: ".concat(params.routeKey) + " - routeIndex: ".concat(params.routeIndex) + " - navStatus: ".concat(this.navStatus) + " - current activeRoutes: ".concat(this.state.activeRoutes.length) + " - current routesToRemove: ".concat(this.routesToRemove.length) + " - activeRoutes: ".concat(JSON.stringify(this.state.activeRoutes))); //#endregion
        // delay, so `routesToRemove` queue gets filled first

        await Helpers.timeout(200); // make a copy of `routesToRemove` and then clear original

        const toBeRemoved = [...this.routesToRemove];
        this.routesToRemove = []; // Remove `routesToRemove` from `activeRoutes`.

        await Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: prevRoutes.filter(prevRoute => !toBeRemoved.some(removedRoute => removedRoute.routeKey === prevRoute.routeKey && removedRoute.routeIndex === prevRoute.routeIndex))
        }));
      }

      ;
      this.setNavStatus(NavStatus.IDLE);
      this.queue.dequeue();
    });

    _defineProperty(this, "_handleGetRefToNavigator", () => {
      return this;
    });

    _defineProperty(this, "getActiveRoutes", () => {
      return this.state.activeRoutes;
    });

    _defineProperty(this, "push", async (routeItem, options) => {
      const stateSnapshot = this.createStateSnapshot();
      const routeConfig = this.getRouteConfig(routeItem.routeKey);

      if (!routeConfig) {
        // no matching route config found for `routeItem`
        throw new Error("NavigatorView' failed to do: 'push', invalid 'routeKey': ".concat(routeItem.routeKey));
      }

      ;

      try {
        var _routeConfig$isNative, _options$isAnimated;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        const {
          activeRoutes
        } = this.state;
        this.setNavStatus(NavStatus.NAV_PUSHING);
        stateSnapshot.save();
        const transitionConfig = this.configureTransitionOverride({
          isPushing: true,
          pushConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
        }); // the amount of time to wait for "push" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily override the last route's "push" transition

        await transitionConfig.setTransition(); //#region - 🐞 DEBUG 🐛

        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, push: add route" + " - with routeKey: ".concat(routeItem.routeKey) + " - current activeRoutes: ".concat(this.state.activeRoutes.length)); //#endregion

        const nextRouteIndex = activeRoutes.length;
        const nextRoute = {
          routeID: ROUTE_ID_COUNTER++,
          routeIndex: nextRouteIndex,
          routeKey: routeItem.routeKey,
          routeProps: routeItem.routeProps,
          routeOptions: routeItem.routeOptions,
          isNativeRoute: (_routeConfig$isNative = routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) !== null && _routeConfig$isNative !== void 0 ? _routeConfig$isNative : false
        };
        await Promise.all([// 1. Wait for new route to be added/init
        this.waitForRoutes([nextRoute]), // 2. Append new route to `activeRoutes`
        Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: [...prevRoutes, nextRoute]
        }))]); // forward "push" request to native module

        await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.push((0, _reactNative.findNodeHandle)(this.nativeRef), nextRoute.routeID, {
          isAnimated: (_options$isAnimated = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated !== void 0 ? _options$isAnimated : true
        })); // reset transition override

        transitionConfig.resetTransition(); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'push' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "pop", async options => {
      const {
        activeRoutes
      } = this.state;
      const stateSnapshot = this.createStateSnapshot();

      if (activeRoutes.length < 1) {
        throw new Error("'pop' failed, active route count must be > 1");
      }

      ;

      try {
        var _options$isAnimated2;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_POPPING);
        stateSnapshot.save();
        const transitionConfig = this.configureTransitionOverride({
          isPushing: false,
          popConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
        }); // the amount of time to wait for "pop" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily change the last route's "pop" transition

        await transitionConfig.setTransition(); // forward "pop" request to native module

        const result = await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.pop((0, _reactNative.findNodeHandle)(this.nativeRef), {
          isAnimated: (_options$isAnimated2 = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated2 !== void 0 ? _options$isAnimated2 : true
        })); // remove popped route from `activeRoutes`

        await Helpers.setStateAsync(this, prevState => ({
          activeRoutes: prevState.activeRoutes.filter(route => !(route.routeIndex === result.routeIndex && route.routeKey === result.routeKey))
        })); // reset transition override

        await transitionConfig.resetTransition();
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'pop' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "popToRoot", async options => {
      const {
        activeRoutes
      } = this.state;
      const stateSnapshot = this.createStateSnapshot();

      if (activeRoutes.length < 1) {
        throw new Error("`popToRoot` failed, route count must be > 1");
      }

      ;

      try {
        var _options$isAnimated3;

        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_POPPING);
        stateSnapshot.save();
        const transitionConfig = this.configureTransitionOverride({
          isPushing: false,
          popConfig: options === null || options === void 0 ? void 0 : options.transitionConfig
        }); // the amount of time to wait for "pop" to resolve before rejecting.

        const timeout = Math.max(transitionConfig.transitionDuration, TIMEOUT_COMMAND); // temporarily change the last route's "pop" transition

        await transitionConfig.setTransition(); // forward `popToRoot` request to native module

        await Helpers.promiseWithTimeout(timeout, _RNINavigatorViewModule.RNINavigatorViewModule.popToRoot((0, _reactNative.findNodeHandle)(this.nativeRef), {
          isAnimated: (_options$isAnimated3 = options === null || options === void 0 ? void 0 : options.isAnimated) !== null && _options$isAnimated3 !== void 0 ? _options$isAnimated3 : true
        })); // remove popped route from `activeRoutes`

        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: [prevState.activeRoutes[0]]
        })); // reset transition override

        await transitionConfig.resetTransition();
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'popToRoot' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "removeRoute", async (routeIndex, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const stateSnapshot = this.createStateSnapshot();
      const routeToBeRemoved = activeRoutes[routeIndex];

      if (activeRoutes.length < 1) {
        throw new Error("`removeRoute` failed, route count must be > 1");
      }

      ;

      if (routeToBeRemoved == null) {
        throw new Error("`removeRoute` failed, invalid index: ".concat(routeIndex));
      }

      ;

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REMOVING);
        stateSnapshot.save();
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.removeRoute((0, _reactNative.findNodeHandle)(this.nativeRef), routeToBeRemoved.routeID, routeToBeRemoved.routeIndex, animated));
        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes // remove route from `activeRoutes`
          .filter(route => route.routeID !== routeToBeRemoved.routeID) // update the route indexes
          .map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'removeRoute' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "removeRoutes", async (routeIndexes, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const stateSnapshot = this.createStateSnapshot();

      if (routeIndexes.length === 0) {
        throw new Error("`removeRoutes` failed, `routeIndexes` is empty");
      }

      ; // check if `routeIndexes` are valid

      for (const routeIndex of routeIndexes) {
        const item = activeRoutes[routeIndex];

        if (item == null) {
          throw new Error("`removeRoutes` failed, invalid index: ".concat(routeIndex));
        }

        ;
      }

      ;

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REMOVING);
        stateSnapshot.save();
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.removeRoutes((0, _reactNative.findNodeHandle)(this.nativeRef), // routes to remove,
        routeIndexes.map(routeIndex => ({
          routeIndex,
          routeID: activeRoutes[routeIndex].routeID
        })), animated));
        await Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes // remove routes from `activeRoutes`
          .filter((_, index) => !routeIndexes.some(routeIndex => routeIndex === index)) // update the route indexes
          .map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'removeRoutes' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "replaceRoute", async (prevRouteIndex, routeItem, animated = false) => {
      var _replacementRouteConf;

      const {
        activeRoutes
      } = this.state;
      const stateSnapshot = this.createStateSnapshot();
      const routeToReplace = activeRoutes[prevRouteIndex];
      const replacementRouteConfig = this.getRouteConfig(routeItem.routeKey);

      if (routeToReplace == null) {
        throw new Error("`replaceRoute` failed, no route found for the given `routeIndex`: ".concat(prevRouteIndex));
      }

      ;

      if (replacementRouteConfig == null) {
        throw new Error("`replaceRoute` failed, no route found for the given `routeKey`: ".concat(routeItem === null || routeItem === void 0 ? void 0 : routeItem.routeKey));
      }

      ;
      const replacementRoute = {
        routeID: ROUTE_ID_COUNTER++,
        routeKey: routeItem.routeKey,
        routeIndex: prevRouteIndex,
        routeProps: routeItem.routeProps,
        routeOptions: routeItem.routeOptions,
        isNativeRoute: (_replacementRouteConf = replacementRouteConfig === null || replacementRouteConfig === void 0 ? void 0 : replacementRouteConfig.isNativeRoute) !== null && _replacementRouteConf !== void 0 ? _replacementRouteConf : false
      };

      try {
        // if busy, wait for prev. to finish first...
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_REPLACING);
        stateSnapshot.save();
        await Promise.all([// 1. wait for replacement route to be added/init
        this.waitForRoutes([replacementRoute]), // 2. replace route in state
        Helpers.setStateAsync(this, prevState => ({ ...prevState,
          activeRoutes: prevState.activeRoutes.map((route, index) => routeToReplace.routeID === route.routeID ? {
            routeIndex: index,
            ...replacementRoute
          } : {
            routeIndex: index,
            ...route
          })
        }))]);
        await Helpers.timeout(500); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.replaceRoute((0, _reactNative.findNodeHandle)(this.nativeRef), prevRouteIndex, routeToReplace.routeID, replacementRoute.routeID, animated));
        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'removeRoutes' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "insertRoute", async (routeItem, atIndex, animated = false) => {
      var _routeConfig$isNative2;

      const state = this.state;
      const routeConfig = this.getRouteConfig(routeItem.routeKey);

      if (!routeConfig) {
        // no matching route config found for `routeItem`
        throw new Error("`NavigatorView` failed to do: `insertRoute`: Invalid `routeKey`");
      }

      ;

      if (atIndex > state.activeRoutes.length) {
        throw new Error("`NavigatorView` failed to do: `insertRoute`: Invalid `atIndex` (out of bounds)");
      }

      ;
      const nextRoute = {
        routeID: ROUTE_ID_COUNTER++,
        routeKey: routeItem.routeKey,
        routeIndex: atIndex,
        routeProps: routeItem.routeProps,
        routeOptions: routeItem.routeOptions,
        isNativeRoute: (_routeConfig$isNative2 = routeConfig === null || routeConfig === void 0 ? void 0 : routeConfig.isNativeRoute) !== null && _routeConfig$isNative2 !== void 0 ? _routeConfig$isNative2 : false
      };

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_INSERTING); //#region - 🐞 DEBUG 🐛

        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, insertRoute: add route" + " - with routeKey: ".concat(nextRoute.routeKey) + " - current activeRoutes: ".concat(this.state.activeRoutes.length)); //#endregion

        await Promise.all([// 1. wait for next route to be added/init
        this.waitForRoutes([nextRoute]), // 2. append new route to `activeRoutes`
        Helpers.setStateAsync(this, ({
          activeRoutes: prevRoutes
        }) => ({
          activeRoutes: Helpers.arrayInsert(prevRoutes, atIndex, nextRoute).map((route, index) => ({ ...route,
            routeIndex: index
          }))
        }))]); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.insertRoute((0, _reactNative.findNodeHandle)(this.nativeRef), nextRoute.routeID, atIndex, animated)); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'insertRoute' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "setRoutes", async (transform, animated = false) => {
      const stateSnapshot = this.createStateSnapshot();

      try {
        // if busy, wait for prev. to finish
        const queue = this.queue.schedule();
        await queue.promise;
        this.setNavStatus(NavStatus.NAV_UPDATING);
        stateSnapshot.save();
        const currentRoutes = [...this.state.activeRoutes];
        const currentRoutesMap = currentRoutes.reduce((acc, curr) => {
          acc[curr.routeID] = { ...curr
          };
          return acc;
        }, {});
        const transformResult = transform(currentRoutes.map(route => ({
          routeID: route.routeID,
          routeKey: route.routeKey
        })));
        const nextRoutes = transformResult.map((route, index) => {
          var _route$routeID;

          return {
            routeProps: route.routeProps,
            // merge old + new route items
            ...currentRoutesMap[route.routeID],
            ...route,
            // assign new routeIndex
            routeIndex: index,
            // assign a routeID if it doesn't have one yet
            routeID: (_route$routeID = route.routeID) !== null && _route$routeID !== void 0 ? _route$routeID : ROUTE_ID_COUNTER++
          };
        }); // get nextRoutes items that aren't mounted/added yet

        const nextRoutesNew = nextRoutes.filter(newRoute => !currentRoutes.some(currentRoute => currentRoute.routeID === newRoute.routeID)); //#region - 🐞 DEBUG 🐛

        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, setRoutes" + " - current routes: ".concat(currentRoutes) + " - next routes: ".concat(nextRoutes) + " - new routes: ".concat(nextRoutesNew)); //#endregion

        await Promise.all([// 1. wait for the new route items to mount (if any)
        this.waitForRoutes(nextRoutesNew), // 2. update the state route items
        Helpers.setStateAsync(this, {
          activeRoutes: nextRoutes
        })]); // forward command to native module

        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.setRoutes((0, _reactNative.findNodeHandle)(this.nativeRef), nextRoutes.map(route => route.routeID), animated)); // finished, start next item

        this.setNavStatus(NavStatus.IDLE);
        this.queue.dequeue();
        stateSnapshot.clear();
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        stateSnapshot.restore();
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'setRoutes' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "setNavigationBarHidden", async (isHidden, animated) => {
      try {
        await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.setNavigationBarHidden((0, _reactNative.findNodeHandle)(this.nativeRef), isHidden, animated));
      } catch (error) {
        const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
        this.setNavStatus(NavStatus.IDLE_ERROR);
        this.queue.dequeue();

        if (!wasAborted) {
          throw new Error("'NavigatorView' failed to do: 'setNavigationBarHidden' with error: ".concat(error));
        }

        ;
      }

      ;
    });

    _defineProperty(this, "replacePreviousRoute", async (routeItem, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.replaceRoute(lastRouteIndex - 1, routeItem, animated);
    });

    _defineProperty(this, "replaceCurrentRoute", async (routeItem, animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.replaceRoute(lastRouteIndex, routeItem, animated);
    });

    _defineProperty(this, "removePreviousRoute", async (animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      await this.removeRoute(lastRouteIndex - 1, animated);
    });

    _defineProperty(this, "removeAllPrevRoutes", async (animated = false) => {
      const {
        activeRoutes
      } = this.state;
      const lastRouteIndex = activeRoutes.length - 1;
      const routesToRemove = activeRoutes.slice(0, lastRouteIndex).map((_, index) => index);
      await this.removeRoutes(routesToRemove, animated);
    });

    _defineProperty(this, "sendCustomCommandToNative", async (commandKey, commandData = null) => {
      return await _RNINavigatorViewModule.RNINavigatorViewModule.sendCustomCommandToNative((0, _reactNative.findNodeHandle)(this.nativeRef), commandKey, commandData);
    });

    _defineProperty(this, "getNavigatorConstants", async () => {
      try {
        const result = await Helpers.promiseWithTimeout(1000, _RNINavigatorViewModule.RNINavigatorViewModule.getNavigatorConstants((0, _reactNative.findNodeHandle)(this.nativeRef)));
        return result;
      } catch (error) {
        //#region - 🐞 DEBUG 🐛
        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, getNavigatorConstants" + " - error message: ".concat(error)); //#endregion

        throw new Error("`NavigatorView` failed to do: `getNavigatorConstants` - " + error);
      }

      ;
    });

    _defineProperty(this, "_handleOnNavRouteViewAdded", event => {
      if (this.navigatorID !== event.nativeEvent.navigatorID) return; // emit event: nav. route was added to `RNINavigatorView`'s subviews

      this.emitter.emit(NavEvents.onNavRouteViewAdded, event);
    });

    _defineProperty(this, "_handleOnSetNativeRoutes", event => {
      if (this.navigatorID !== event.nativeEvent.navigatorID) return; // emit event: route data was set for the native routes

      this.emitter.emit(NavEvents.onSetNativeRoutes, event);
    });

    _defineProperty(this, "_handleOnNativeCommandRequest", async ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;
      const {
        commandData
      } = nativeEvent; //#region - 🐞 DEBUG 🐛

      LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, _handleOnNativeCommandRequest" + "commandKey: ".concat(nativeEvent.commandData.commandKey) + "commandData: ".concat(JSON.stringify(nativeEvent.commandData))); //#endregion

      switch (commandData.commandKey) {
        // TODO: Move to sep. function
        case 'pushViewController':
          try {
            // if busy, wait for prev. to finish
            const queue = this.queue.schedule();
            await queue.promise;
            const {
              activeRoutes
            } = this.state;
            const nextRouteIndex = activeRoutes.length;
            this.setNavStatus(NavStatus.NAV_PUSHING);
            const nextRoute = {
              routeID: commandData.routeID,
              routeKey: commandData.routeKey,
              routeIndex: nextRouteIndex,
              isNativeRoute: true
            };
            await Promise.all([// 1. wait for the native route to be init
            this.waitForRoutes([nextRoute]), // 2. Append new route to `activeRoutes`
            Helpers.setStateAsync(this, ({
              activeRoutes: prevRoutes
            }) => ({
              activeRoutes: [...prevRoutes, nextRoute]
            }))]); // forward "push" request to native module

            await Helpers.promiseWithTimeout(TIMEOUT_COMMAND, _RNINavigatorViewModule.RNINavigatorViewModule.push((0, _reactNative.findNodeHandle)(this.nativeRef), nextRoute.routeID, {
              isAnimated: commandData.isAnimated
            })); // finished, start next item

            this.setNavStatus(NavStatus.IDLE);
            this.queue.dequeue();
          } catch (error) {
            const wasAborted = NavigatorViewUtils.wasAborted(this.navStatus);
            this.setNavStatus(NavStatus.IDLE_ERROR);
            this.queue.dequeue();

            if (!wasAborted) {
              throw new Error("'NavigatorView' failed to do: '".concat(commandData.commandKey, "'") + "with error: ".concat(error));
            }

            ;
          }

          ;
          break;

        case 'push':
          await this.push({
            routeKey: commandData.routeKey,
            routeProps: commandData.routeProps
          }, {
            isAnimated: commandData.isAnimated
          });
          break;

        case 'pop':
          await this.pop({
            isAnimated: commandData.isAnimated
          });
          break;
      }

      ;
    });

    _defineProperty(this, "_handleOnCustomCommandFromNative", event => {
      var _this$props$onCustomC, _this$props;

      if (this.navigatorID !== event.nativeEvent.navigatorID) return;
      (_this$props$onCustomC = (_this$props = this.props).onCustomCommandFromNative) === null || _this$props$onCustomC === void 0 ? void 0 : _this$props$onCustomC.call(_this$props, event);
    });

    _defineProperty(this, "_handleOnNavRouteWillPop", ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;

      if (!NavigatorViewUtils.isNavStateIdle(this.navStatus)) {
        this.setNavStatus(NavStatus.NAV_ABORT);
      }

      ;
    });

    _defineProperty(this, "_handleOnNavRouteDidPop", ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return; //#region - 🐞 DEBUG 🐛

      LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, onNavRouteDidPop" + " - with args, routeKey: ".concat(nativeEvent.routeKey) + " - routeIndex: ".concat(nativeEvent.routeIndex)); //#endregion
      // A route has been removed either through a tap on the "back" 
      // button, or through a swipe back gesture.

      if (nativeEvent.isUserInitiated) {
        // cleanup - remove route
        this.removeRouteBatchedFromState({
          routeKey: nativeEvent.routeKey,
          routeIndex: nativeEvent.routeIndex
        });
      }

      ;
    });

    _defineProperty(this, "_handleOnUIConstantsDidChange", ({
      nativeEvent
    }) => {
      if (this.navigatorID !== nativeEvent.navigatorID) return;
      this.setState({
        safeAreaInsets: nativeEvent.safeAreaInsets,
        statusBarHeight: nativeEvent.statusBarHeight
      });
    });

    this.navigatorID = NAVIGATOR_ID_COUNTER++;
    this.navStatus = NavStatus.IDLE_INIT;
    this.routesToRemove = [];
    this.emitter = new _EventEmitter.EventEmitter();
    this.queue = new _SimpleQueue.SimpleQueue();
    this.state = {
      activeRoutes: this.getInitialRoutes(),
      transitionConfigPushOverride: null,
      transitionConfigPopOverride: null,
      safeAreaInsets: null,
      statusBarHeight: null
    };
  }

  componentWillUnmount() {
    this.setNavStatus(NavStatus.UNMOUNTED);
    this.queue.clear();
  }

  setNavStatus(navStatus) {
    if (this.navStatus !== NavStatus.UNMOUNTED) {
      //#region - 🐞 DEBUG 🐛
      LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorView, setNavStatus" + " - next status: ".concat(navStatus) + " - current status: ".concat(this.navStatus)); //#endregion

      this.navStatus = navStatus;
    }

    ;
  }

  //#endregion
  _renderRoutes() {
    const props = this.props;
    const state = this.state; // * Get the `state.activeRoutes` sorted by their `routeID`, such that the
    //   routes are ordered from oldest to newest.
    // * Supposedly, when reordering comps, as long as the key of a comp. does
    //   not change, the comps. should be "preserved", but When replacing the 1st 
    //   route (i.e. replacing `state.activeRoutes[0]`), it causes all the other 
    //   routes to disappear.
    // * Specifically, it triggers `componentWillUnmount` for all the other 
    //   active routes (even though there aren't any changes to their props/keys).
    // * This then triggers `vc.didMove(toParent: nil`) in the native side,
    //   which then triggers the cleanup/remove process.
    // * This bug has something to do with the re-ordering of components, causing
    //   the comps. to be "recreated" from scratch.
    // * So this is a temp. workaround to prevent the unnecessary reordering of the
    //   comps., the downside is that the routes need to be sorted on every render...

    const activeRoutes = this.getRoutesToRender();
    const activeRoutesCount = activeRoutes.length;
    const activeRoutesLastIndex = activeRoutesCount - 1;
    return activeRoutes.map(route => {
      // the routes from `activeRoutes` will only ever have js/react routes
      const routeConfig = this.getRouteConfig(route.routeKey);
      const isLast = route.routeIndex === activeRoutesLastIndex;
      const isSecondToLast = route.routeIndex === activeRoutesLastIndex - 1;
      return /*#__PURE__*/_react.default.createElement(_NavigatorRouteView.NavigatorRouteView, {
        key: "routeID-".concat(route.routeID),
        ref: r => {
          this.routeRefMap[route.routeID] = r;
        },
        navigatorID: this.navigatorID,
        routeID: route.routeID,
        routeIndex: route.routeIndex,
        routeKey: route.routeKey,
        isRootRoute: route.routeIndex === 0,
        currentActiveRouteIndex: activeRoutesLastIndex // merge routeProps
        ,
        routeProps: Helpers.shallowMergeObjects(routeConfig.initialRouteProps, route.routeProps) // merge routeOptions
        ,
        routeOptionsDefault: Helpers.shallowMergeObjects(routeConfig.routeOptionsDefault, route.routeOptions),
        transitionConfigPushOverride: isSecondToLast ? state.transitionConfigPushOverride : null,
        transitionConfigPopOverride: isLast ? state.transitionConfigPopOverride : null,
        getRefToNavigator: this._handleGetRefToNavigator,
        renderRouteContent: () => routeConfig.renderRoute(route),
        renderNavBarLeftItem: params => {
          var _routeConfig$renderNa, _routeConfig$renderNa2, _props$renderNavBarLe;

          return (_routeConfig$renderNa = (_routeConfig$renderNa2 = routeConfig.renderNavBarLeftItem) === null || _routeConfig$renderNa2 === void 0 ? void 0 : _routeConfig$renderNa2.call(routeConfig, params)) !== null && _routeConfig$renderNa !== void 0 ? _routeConfig$renderNa : (_props$renderNavBarLe = props.renderNavBarLeftItem) === null || _props$renderNavBarLe === void 0 ? void 0 : _props$renderNavBarLe.call(props, params);
        },
        renderNavBarRightItem: params => {
          var _routeConfig$renderNa3, _routeConfig$renderNa4, _props$renderNavBarRi;

          return (_routeConfig$renderNa3 = (_routeConfig$renderNa4 = routeConfig.renderNavBarRightItem) === null || _routeConfig$renderNa4 === void 0 ? void 0 : _routeConfig$renderNa4.call(routeConfig, params)) !== null && _routeConfig$renderNa3 !== void 0 ? _routeConfig$renderNa3 : (_props$renderNavBarRi = props.renderNavBarRightItem) === null || _props$renderNavBarRi === void 0 ? void 0 : _props$renderNavBarRi.call(props, params);
        },
        renderNavBarTitleItem: params => {
          var _routeConfig$renderNa5, _routeConfig$renderNa6, _props$renderNavBarTi;

          return (_routeConfig$renderNa5 = (_routeConfig$renderNa6 = routeConfig.renderNavBarTitleItem) === null || _routeConfig$renderNa6 === void 0 ? void 0 : _routeConfig$renderNa6.call(routeConfig, params)) !== null && _routeConfig$renderNa5 !== void 0 ? _routeConfig$renderNa5 : (_props$renderNavBarTi = props.renderNavBarTitleItem) === null || _props$renderNavBarTi === void 0 ? void 0 : _props$renderNavBarTi.call(props, params);
        }
      });
    });
  }

  render() {
    var _props$isInteractiveP, _props$shouldSwizzleR, _props$isNavBarTransl, _props$navBarPrefersL;

    const props = this.props;
    const state = this.state;
    return /*#__PURE__*/_react.default.createElement(_RNINavigatorView.RNINavigatorView, {
      ref: r => {
        this.nativeRef = r;
      },
      style: [styles.navigatorView, props.style] // General config
      ,
      navigatorID: this.navigatorID,
      isInteractivePopGestureEnabled: (_props$isInteractiveP = props.isInteractivePopGestureEnabled) !== null && _props$isInteractiveP !== void 0 ? _props$isInteractiveP : true,
      shouldSwizzleRootViewController: (_props$shouldSwizzleR = props.shouldSwizzleRootViewController) !== null && _props$shouldSwizzleR !== void 0 ? _props$shouldSwizzleR : true,
      nativeRoutes: this.getNativeRoutes(),
      initialRouteKeys: props.initialRoutes.map(route => route.routeKey) // Navigation Bar customization
      ,
      isNavBarTranslucent: (_props$isNavBarTransl = props.isNavBarTranslucent) !== null && _props$isNavBarTransl !== void 0 ? _props$isNavBarTransl : true,
      navBarPrefersLargeTitles: (_props$navBarPrefersL = props.navBarPrefersLargeTitles) !== null && _props$navBarPrefersL !== void 0 ? _props$navBarPrefersL : true,
      navBarAppearance: props.navBarAppearance // event handlers: push/pop
      ,
      onNavRouteWillPop: this._handleOnNavRouteWillPop,
      onNavRouteDidPop: this._handleOnNavRouteDidPop,
      onNavRouteViewAdded: this._handleOnNavRouteViewAdded,
      onSetNativeRoutes: this._handleOnSetNativeRoutes,
      onNativeCommandRequest: this._handleOnNativeCommandRequest,
      onCustomCommandFromNative: this._handleOnCustomCommandFromNative,
      onUIConstantsDidChange: this._handleOnUIConstantsDidChange
    }, /*#__PURE__*/_react.default.createElement(_NavigatorUIConstantsContext.NavigatorUIConstantsContext.Provider, {
      value: {
        safeAreaInsets: state.safeAreaInsets,
        statusBarHeight: state.statusBarHeight
      }
    }, this._renderRoutes(), props.renderNavBarBackground && /*#__PURE__*/_react.default.createElement(_RNIWrapperView.RNIWrapperView, {
      style: styles.navBarBackgroundContainer,
      nativeID: _LibraryConstants.NativeIDKeys.NavBarBackground
    }, props.renderNavBarBackground())));
  }

}

exports.NavigatorView = NavigatorView;
;
/** Utilities for `NavigatorView` */

class NavigatorViewUtils {
  static sortByRouteIndex(routes) {
    return routes.sort((a, b) => a.routeIndex - b.routeIndex);
  }

  static isNavStateIdle(navStatus) {
    return navStatus === NavStatus.IDLE || navStatus === NavStatus.IDLE_INIT || navStatus === NavStatus.IDLE_ERROR || navStatus === NavStatus.NAV_ABORT;
  }

  static wasAborted(navStatus) {
    return navStatus === NavStatus.UNMOUNTED || navStatus === NavStatus.NAV_ABORT || navStatus === NavStatus.NAV_POPPING;
  }

  static isNavStateBusy(navStatus) {
    return !NavigatorViewUtils.isNavStateIdle(navStatus);
  }

  static getNativeRouteKeys() {
    return new Promise(resolve => {
      _RNINavigatorViewModule.RNINavigatorViewModule.getNativeRouteKeys(keys => {
        resolve(keys.reduce((acc, curr) => {
          acc[curr] = curr;
          return acc;
        }, {}));
      });
    });
  }

}

; // init. `nativeRouteKeys`

NavigatorViewUtils.getNativeRouteKeys().then(keys => {
  nativeRouteKeys = keys;
});

const styles = _reactNative.StyleSheet.create({
  navigatorView: {
    flex: 1
  },
  navBarBackgroundContainer: {
    position: 'absolute'
  }
});
//# sourceMappingURL=NavigatorView.js.map