"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EventEmitter = void 0;

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

//#region - Type Definitions

/** shim that tries to match the ts's builtin `enum` type */
//#endregion
class EventEmitter {
  // Properties

  /** Store the event listeners */
  constructor() {
    _defineProperty(this, "events", void 0);

    this.events = {};
  }

  addListener(eventKey, listener) {
    const hasEvents = this.events[eventKey] != null;

    if (!hasEvents) {
      // initialize with empty array...
      // The array will be used to store the event listeners
      this.events[eventKey] = [];
    }

    ;
    this.events[eventKey].push(listener);
  }

  removeListener(eventKey, listenerToRemove) {
    const hasEvents = this.events[eventKey] != null; // event does not exist (maybe: throw an error?)

    if (!hasEvents) return;
    this.events[eventKey] = this.events[eventKey].filter(listener => listener !== listenerToRemove);
  }

  once(eventKey, listener) {
    const tempListener = data => {
      listener(data);
      this.removeListener(eventKey, tempListener);
    };

    this.addListener(eventKey, tempListener);
  }

  removeAllListeners() {
    this.events = {};
  }

  emit(eventKey, data) {
    const hasEvents = this.events[eventKey] != null; // event does not exist

    if (!hasEvents) return;
    this.events[eventKey].forEach(callback => {
      callback(data);
    });
  }

}

exports.EventEmitter = EventEmitter;
;
//# sourceMappingURL=EventEmitter.js.map