"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useNavRouteLifeCycle = useNavRouteLifeCycle;
exports.useNavBarItemEvents = useNavBarItemEvents;

var _react = require("react");

var _NavRouteViewContext = require("../context/NavRouteViewContext");

function useNavRouteEvents(eventName, handler) {
  const {
    navigation
  } = (0, _react.useContext)(_NavRouteViewContext.NavRouteViewContext); // Create a ref that stores handler

  const savedHandler = (0, _react.useRef)(null); // Update ref.current value if handler changes.
  // This allows our effect below to always get latest handler ...
  // ... without us needing to pass it in effect deps array ...
  // ... and potentially cause effect to re-run every render.

  (0, _react.useEffect)(() => {
    savedHandler.current = handler;
  }, [handler]);
  (0, _react.useEffect)(() => {
    const emitterRef = navigation.getRefToNavRouteEmitter(); // create event listener that calls handler function stored in ref

    const eventListener = params => savedHandler.current(params); // subscribe to events


    emitterRef.addListener(eventName, eventListener);
    return () => {
      // remove event listener on cleanup
      emitterRef.removeListener(eventName, eventListener);
    };
  });
}

;

function useNavRouteLifeCycle(eventName, handler) {
  useNavRouteEvents(eventName, handler);
}

;

function useNavBarItemEvents(eventName, handler) {
  useNavRouteEvents(eventName, handler);
}

;
//# sourceMappingURL=useNavRouteEvents.js.map