function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import React from 'react';
import { StyleSheet, View, findNodeHandle } from 'react-native';
import { RouteComponentsWrapper } from './RouteComponentsWrapper';
import { RNINavigatorRouteView } from '../native_components/RNINavigatorRouteView';
import { RNINavigatorRouteViewModule } from '../native_modules/RNINavigatorRouteViewModule';
import * as Helpers from '../functions/Helpers';
import { EventEmitter } from '../functions/EventEmitter';
import { CompareRouteTransitionPushConfig } from '../functions/CompareRouteOptions';
import { NavRouteViewContext } from '../context/NavRouteViewContext';
import { NavigatorUIConstantsContext } from '../context/NavigatorUIConstantsContext';
import { NativeIDKeys } from '../constants/LibraryConstants'; //#region - Type Definitions

/** Event emitter keys for `NavigatorRouteView` */

export let NavRouteEvents;

(function (NavRouteEvents) {
  NavRouteEvents["onRouteWillPush"] = "onRouteWillPush";
  NavRouteEvents["onRouteDidPush"] = "onRouteDidPush";
  NavRouteEvents["onRouteWillPop"] = "onRouteWillPop";
  NavRouteEvents["onRouteDidPop"] = "onRouteDidPop";
  NavRouteEvents["onRouteWillFocus"] = "onRouteWillFocus";
  NavRouteEvents["onRouteDidFocus"] = "onRouteDidFocus";
  NavRouteEvents["onRouteWillBlur"] = "onRouteWillBlur";
  NavRouteEvents["onRouteDidBlur"] = "onRouteDidBlur";
  NavRouteEvents["onPressNavBarLeftItem"] = "onPressNavBarLeftItem";
  NavRouteEvents["onPressNavBarRightItem"] = "onPressNavBarRightItem";
  NavRouteEvents["onUpdateSearchResults"] = "onUpdateSearchResults";
  NavRouteEvents["onSearchBarCancelButtonClicked"] = "onSearchBarCancelButtonClicked";
  NavRouteEvents["onSearchBarSearchButtonClicked"] = "onSearchBarSearchButtonClicked";
})(NavRouteEvents || (NavRouteEvents = {}));

;
;
var RouteStatus;

(function (RouteStatus) {
  RouteStatus["INIT"] = "INIT";
  RouteStatus["ROUTE_PUSHING"] = "ROUTE_PUSHING";
  RouteStatus["ROUTE_PUSHED"] = "ROUTE_PUSHED";
  RouteStatus["ROUTE_POPPING"] = "ROUTE_POPPING";
  RouteStatus["ROUTE_POPPED"] = "ROUTE_POPPED";
  RouteStatus["ROUTE_BLURRING"] = "ROUTE_BLURRING";
  RouteStatus["ROUTE_BLURRED"] = "ROUTE_BLURRED";
  RouteStatus["ROUTE_FOCUSING"] = "ROUTE_FOCUSING";
  RouteStatus["ROUTE_FOCUSED"] = "ROUTE_FOCUSED";
  RouteStatus["UNMOUNTED"] = "UNMOUNTED";
})(RouteStatus || (RouteStatus = {}));

;
//#endregion
export class NavigatorRouteView extends React.Component {
  //#region - Property Declarations
  // references
  //#endregion
  constructor(_props) {
    super(_props);

    _defineProperty(this, "state", void 0);

    _defineProperty(this, "context", void 0);

    _defineProperty(this, "routeContentRef", void 0);

    _defineProperty(this, "routeStatus", void 0);

    _defineProperty(this, "_emitter", void 0);

    _defineProperty(this, "_nativeRef", void 0);

    _defineProperty(this, "_navigatorRef", void 0);

    _defineProperty(this, "_routeContentRef", void 0);

    _defineProperty(this, "_routeViewPortalRef", void 0);

    _defineProperty(this, "_routeComponentsWrapperRef", void 0);

    _defineProperty(this, "getRefToNavigator", () => {
      return this.props.getRefToNavigator();
    });

    _defineProperty(this, "getRouteRef", () => {
      return this;
    });

    _defineProperty(this, "getEmitterRef", () => {
      return this._emitter;
    });

    _defineProperty(this, "getPortalRef", () => {
      return this._routeViewPortalRef;
    });

    _defineProperty(this, "getRouteComponentsWrapper", () => {
      return this._routeComponentsWrapperRef;
    });

    _defineProperty(this, "getRouteOptions", () => {
      var _this$_routeViewPorta, _routeOptions$statusB, _routeOptions$routeCo, _ref, _routeOptions$automat, _ref2, _props$transitionConf, _ref3, _props$transitionConf2, _ref4, _routeOptions$routeTi, _routeOptions$prompt, _routeOptions$largeTi, _routeOptions$searchB, _routeOptions$navBarA, _routeOptions$navigat, _routeOptions$allowTo, _routeOptions$navBarB, _ref5, _routeOptions$navBarB2, _ref6, _routeOptions$navBarB3, _routeOptions$leftIte, _routeOptions$backBut, _routeOptions$backBut2, _routeOptions$hidesBa, _routeOptions$applyBa;

      const props = this.props;
      const portalProps = (_this$_routeViewPorta = this._routeViewPortalRef) === null || _this$_routeViewPorta === void 0 ? void 0 : _this$_routeViewPorta.props;
      const {
        routeOptionsDefault
      } = this.props;
      const {
        routeOptions
      } = this.state; // check if portal has custom nav bar items

      const hasNavBarLeftItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarLeftItem) != null;
      const hasNavBarRightItem = (portalProps === null || portalProps === void 0 ? void 0 : portalProps.renderNavBarRightItem) != null;
      return {
        // ------------------------------------------------------
        // Technically, this whole thing could be done like this:
        // `{...routeOptionsDefault, ...routeOptions}`
        // but it's less clear/explicit, idk refactor this later.
        // ------------------------------------------------------
        statusBarStyle: (_routeOptions$statusB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.statusBarStyle) !== null && _routeOptions$statusB !== void 0 ? _routeOptions$statusB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.statusBarStyle,
        routeContainerStyle: (_routeOptions$routeCo = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.routeContainerStyle) !== null && _routeOptions$routeCo !== void 0 ? _routeOptions$routeCo : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeContainerStyle,
        automaticallyAddHorizontalSafeAreaInsets: (_ref = (_routeOptions$automat = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.automaticallyAddHorizontalSafeAreaInsets) !== null && _routeOptions$automat !== void 0 ? _routeOptions$automat : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.automaticallyAddHorizontalSafeAreaInsets) !== null && _ref !== void 0 ? _ref : true,
        // #region - Transition Config |
        // ----------------------------*
        transitionConfigPush: (_ref2 = (_props$transitionConf = props.transitionConfigPushOverride) !== null && _props$transitionConf !== void 0 ? _props$transitionConf : routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.transitionConfigPush) !== null && _ref2 !== void 0 ? _ref2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPush,
        transitionConfigPop: (_ref3 = (_props$transitionConf2 = props.transitionConfigPopOverride) !== null && _props$transitionConf2 !== void 0 ? _props$transitionConf2 : routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.transitionConfigPop) !== null && _ref3 !== void 0 ? _ref3 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.transitionConfigPop,
        // #endregion -------------*
        // #region - Navbar Config |
        // ------------------------*
        routeTitle: (_ref4 = (_routeOptions$routeTi = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.routeTitle) !== null && _routeOptions$routeTi !== void 0 ? _routeOptions$routeTi : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.routeTitle) !== null && _ref4 !== void 0 ? _ref4 : props.routeKey,
        prompt: (_routeOptions$prompt = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.prompt) !== null && _routeOptions$prompt !== void 0 ? _routeOptions$prompt : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.prompt,
        largeTitleDisplayMode: (_routeOptions$largeTi = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.largeTitleDisplayMode) !== null && _routeOptions$largeTi !== void 0 ? _routeOptions$largeTi : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.largeTitleDisplayMode,
        searchBarConfig: (_routeOptions$searchB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.searchBarConfig) !== null && _routeOptions$searchB !== void 0 ? _routeOptions$searchB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.searchBarConfig,
        // #endregion ----------------------------------*
        // #region - `NavigationConfigOverride`-related |
        // ---------------------------------------------*
        navBarAppearanceOverride: (_routeOptions$navBarA = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarAppearanceOverride) !== null && _routeOptions$navBarA !== void 0 ? _routeOptions$navBarA : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarAppearanceOverride,
        navigationBarVisibility: (_routeOptions$navigat = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navigationBarVisibility) !== null && _routeOptions$navigat !== void 0 ? _routeOptions$navigat : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navigationBarVisibility,
        allowTouchEventsToPassThroughNavigationBar: (_routeOptions$allowTo = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.allowTouchEventsToPassThroughNavigationBar) !== null && _routeOptions$allowTo !== void 0 ? _routeOptions$allowTo : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.allowTouchEventsToPassThroughNavigationBar,
        // #endregion ------------------*
        // #region - Navbar Item Config |
        // -----------------------------*
        navBarButtonBackItemConfig: (_routeOptions$navBarB = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonBackItemConfig) !== null && _routeOptions$navBarB !== void 0 ? _routeOptions$navBarB : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonBackItemConfig,
        navBarButtonLeftItemsConfig: (_ref5 = (_routeOptions$navBarB2 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonLeftItemsConfig) !== null && _routeOptions$navBarB2 !== void 0 ? _routeOptions$navBarB2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonLeftItemsConfig) !== null && _ref5 !== void 0 ? _ref5 : // custom left bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarLeftItem ? [{
          type: 'CUSTOM'
        }] : null,
        navBarButtonRightItemsConfig: (_ref6 = (_routeOptions$navBarB3 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.navBarButtonRightItemsConfig) !== null && _routeOptions$navBarB3 !== void 0 ? _routeOptions$navBarB3 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.navBarButtonRightItemsConfig) !== null && _ref6 !== void 0 ? _ref6 : // custom right bar item was set, so we implicitly/automatically
        // create a `type: CUSTOM` nav bar item config...
        hasNavBarRightItem ? [{
          type: 'CUSTOM'
        }] : null,
        // #endregion ------------------------------*
        // #region - Navbar back button item config |
        // -----------------------------------------*
        leftItemsSupplementBackButton: (_routeOptions$leftIte = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.leftItemsSupplementBackButton) !== null && _routeOptions$leftIte !== void 0 ? _routeOptions$leftIte : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.leftItemsSupplementBackButton,
        backButtonTitle: (_routeOptions$backBut = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.backButtonTitle) !== null && _routeOptions$backBut !== void 0 ? _routeOptions$backBut : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonTitle,
        backButtonDisplayMode: (_routeOptions$backBut2 = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.backButtonDisplayMode) !== null && _routeOptions$backBut2 !== void 0 ? _routeOptions$backBut2 : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.backButtonDisplayMode,
        hidesBackButton: (_routeOptions$hidesBa = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.hidesBackButton) !== null && _routeOptions$hidesBa !== void 0 ? _routeOptions$hidesBa : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.hidesBackButton,
        applyBackButtonConfigToCurrentRoute: (_routeOptions$applyBa = routeOptions === null || routeOptions === void 0 ? void 0 : routeOptions.applyBackButtonConfigToCurrentRoute) !== null && _routeOptions$applyBa !== void 0 ? _routeOptions$applyBa : routeOptionsDefault === null || routeOptionsDefault === void 0 ? void 0 : routeOptionsDefault.applyBackButtonConfigToCurrentRoute // #endregion

      };
    });

    _defineProperty(this, "getRouteNavigationObject", () => {
      const props = this.props;
      const routeOptions = this.getRouteOptions();
      return {
        routeOptions,
        // pass down route details/data
        routeKey: props.routeKey,
        routeIndex: props.routeIndex,
        routeProps: props.routeProps,
        // pass down navigator commands
        push: this._navigatorRef.push,
        pop: this._navigatorRef.pop,
        popToRoot: this._navigatorRef.popToRoot,
        removeRoute: this._navigatorRef.removeRoute,
        removeRoutes: this._navigatorRef.removeRoutes,
        replaceRoute: this._navigatorRef.replaceRoute,
        insertRoute: this._navigatorRef.insertRoute,
        setRoutes: this._navigatorRef.setRoutes,
        setNavigationBarHidden: this._navigatorRef.setNavigationBarHidden,
        // pass down misc. navigator commands
        sendCustomCommandToNative: this._navigatorRef.sendCustomCommandToNative,
        getNavigatorConstants: this._navigatorRef.getNavigatorConstants,
        // pass down convenience navigator commands
        replacePreviousRoute: this._navigatorRef.replacePreviousRoute,
        replaceCurrentRoute: this._navigatorRef.replaceCurrentRoute,
        removePreviousRoute: this._navigatorRef.removePreviousRoute,
        removeAllPrevRoutes: this._navigatorRef.removeAllPrevRoutes,
        // navigator route commands
        getRouteOptions: this.getRouteOptions,
        setRouteOptions: this.setRouteOptions,
        setHidesBackButton: this.setHidesBackButton,
        getRouteConstants: this.getRouteConstants,
        // pass down 'get ref' functions
        getRefToRoute: this._handleGetRefToRoute,
        getRefToNavigator: props.getRefToNavigator,
        getRefToNavRouteEmitter: this._handleGetRefToNavRouteEmitter
      };
    });

    _defineProperty(this, "setRouteOptions", async routeOptions => {
      await Helpers.setStateAsync(this, prevState => ({ ...prevState,
        routeOptions,
        updateIndex: prevState.updateIndex + 1
      }));
    });

    _defineProperty(this, "setRouteViewPortalRef", ref => {
      var _this$_routeComponent;

      this._routeViewPortalRef = ref;
      (_this$_routeComponent = this._routeComponentsWrapperRef) === null || _this$_routeComponent === void 0 ? void 0 : _this$_routeComponent.setPortalRef(ref);
      this.setState({
        hasRoutePortal: true
      });
    });

    _defineProperty(this, "setHidesBackButton", async (isHidden, animated) => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new Error("`NavigatorRouteView` is not mounted");
        }

        ;
        await Helpers.promiseWithTimeout(1000, RNINavigatorRouteViewModule.setHidesBackButton(findNodeHandle(this._nativeRef), isHidden, animated));
      } catch (error) {
        //#region - 🐞 DEBUG 🐛
        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorRouteView, setHidesBackButton" + " - error message: ".concat(error)); //#endregion

        throw new Error("`NavigatorRouteView` failed to do: `setHidesBackButton` - " + error);
      }

      ;
    });

    _defineProperty(this, "getRouteConstants", async () => {
      try {
        if (!RouteViewUtils.isRouteReady(this.routeStatus)) {
          throw new Error("`NavigatorRouteView` is not mounted");
        }

        ;
        const result = await Helpers.promiseWithTimeout(1000, RNINavigatorRouteViewModule.getRouteConstants(findNodeHandle(this._nativeRef)));
        return result;
      } catch (error) {
        //#region - 🐞 DEBUG 🐛
        LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorRouteView, getConstants" + " - error message: ".concat(error)); //#endregion

        throw new Error("`NavigatorRouteView` failed to do: `getConstants` - " + error);
      }

      ;
    });

    _defineProperty(this, "_handleGetRefToRoute", () => {
      return this;
    });

    _defineProperty(this, "_handleGetRefToNavRouteEmitter", () => {
      return this._emitter;
    });

    _defineProperty(this, "_handleOnNavRouteWillPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteWillPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPING;
    });

    _defineProperty(this, "_handleOnNavRouteDidPop", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteDidPop, event);

      this.routeStatus = RouteStatus.ROUTE_POPPED;
    });

    _defineProperty(this, "_handleOnNavRouteWillPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteWillPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHING;
    });

    _defineProperty(this, "_handleOnNavRouteDidPush", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteDidPush, event);

      this.routeStatus = RouteStatus.ROUTE_PUSHED;
    });

    _defineProperty(this, "_handleOnPressNavBarLeftItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onPressNavBarLeftItem, event);
    });

    _defineProperty(this, "_handleOnPressNavBarRightItem", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onPressNavBarRightItem, event);
    });

    _defineProperty(this, "_handleOnRouteWillFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteWillFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSING;
    });

    _defineProperty(this, "_handleOnRouteDidFocus", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteDidFocus, event);

      this.routeStatus = RouteStatus.ROUTE_FOCUSED;
    });

    _defineProperty(this, "_handleOnRouteWillBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteWillBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRING;
    });

    _defineProperty(this, "_handleOnRouteDidBlur", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onRouteDidBlur, event);

      this.routeStatus = RouteStatus.ROUTE_BLURRED;
    });

    _defineProperty(this, "_handleOnUpdateSearchResults", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onUpdateSearchResults, event);
    });

    _defineProperty(this, "_handleOnSearchBarCancelButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onSearchBarCancelButtonClicked, event);
    });

    _defineProperty(this, "_handleOnSearchBarSearchButtonClicked", event => {
      if (this.props.routeID !== event.nativeEvent.routeID) return;

      this._emitter.emit(NavRouteEvents.onSearchBarSearchButtonClicked, event);
    });

    _defineProperty(this, "_renderRouteContents", navigation => {
      var _this$_routeViewPorta2, _portalProps$renderRo, _portalProps$renderRo2, _props$renderRouteHea, _safeAreaInsets$left, _safeAreaInsets$right;

      const props = this.props;
      const context = this.context;
      const routeContent = props.renderRouteContent();
      const routeOptions = navigation.routeOptions;
      const safeAreaInsets = context.safeAreaInsets;
      const routeContentWithProps = /*#__PURE__*/React.cloneElement(routeContent, {
        navigation,
        // store a ref to this element
        ...(Helpers.isClassComponent(routeContent) && {
          ref: node => {
            this._routeContentRef = node;
          }
        })
      });
      const portalProps = (_this$_routeViewPorta2 = this._routeViewPortalRef) === null || _this$_routeViewPorta2 === void 0 ? void 0 : _this$_routeViewPorta2.props;
      const routeHeader = (_portalProps$renderRo = portalProps === null || portalProps === void 0 ? void 0 : (_portalProps$renderRo2 = portalProps.renderRouteHeader) === null || _portalProps$renderRo2 === void 0 ? void 0 : _portalProps$renderRo2.call(portalProps, navigation)) !== null && _portalProps$renderRo !== void 0 ? _portalProps$renderRo : (_props$renderRouteHea = props.renderRouteHeader) === null || _props$renderRouteHea === void 0 ? void 0 : _props$renderRouteHea.call(props, navigation);
      const routeContainerStyle = { ...(routeOptions.automaticallyAddHorizontalSafeAreaInsets && {
          paddingLeft: (_safeAreaInsets$left = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.left) !== null && _safeAreaInsets$left !== void 0 ? _safeAreaInsets$left : 0,
          paddingRight: (_safeAreaInsets$right = safeAreaInsets === null || safeAreaInsets === void 0 ? void 0 : safeAreaInsets.right) !== null && _safeAreaInsets$right !== void 0 ? _safeAreaInsets$right : 0
        })
      };
      return /*#__PURE__*/React.createElement(View, {
        style: [styles.routeContentContainer, routeContainerStyle, routeOptions.routeContainerStyle],
        nativeID: NativeIDKeys.RouteContent
      }, routeContentWithProps, routeHeader);
    });

    this.routeStatus = RouteStatus.INIT;
    this._emitter = new EventEmitter();
    this._navigatorRef = _props.getRefToNavigator();
    this.state = {
      updateIndex: 0,
      hasRoutePortal: false,
      routeOptions: {}
    };
  }

  shouldComponentUpdate(nextProps, nextState) {
    const prevProps = this.props;
    const prevState = this.state;
    const routeIndex = prevProps.routeIndex;
    return (// props: compare `routeIndex` - routes were re-arranged/sorted
      prevProps.routeIndex !== nextProps.routeIndex || // props: compare focus - topmost route <-> not topmost route
      routeIndex === prevProps.currentActiveRouteIndex !== (routeIndex === nextProps.currentActiveRouteIndex) // props: compare "push transition config"
      || CompareRouteTransitionPushConfig.compare(prevProps.transitionConfigPushOverride, nextProps.transitionConfigPushOverride) // props: compare "pop transition config"
      || CompareRouteTransitionPushConfig.compare(prevProps.transitionConfigPushOverride, nextProps.transitionConfigPushOverride) // state: compare `hasRoutePortal` - portal comp. added/removed 
      || prevState.hasRoutePortal !== nextState.hasRoutePortal // state: compare `updateIndex` - force an update
      || prevState.updateIndex !== nextState.updateIndex
    );
  }

  componentWillUnmount() {
    //#region - 🐞 DEBUG 🐛
    LIB_GLOBAL.debugLog && console.log("LOG/JS - NavigatorRouteView, componentWillUnmount" + " - routeID: ".concat(this.props.routeID) + " - routeIndex: ".concat(this.props.routeIndex) + " - routeKey: ".concat(this.props.routeKey)); //#endregion

    this.routeStatus = RouteStatus.UNMOUNTED;
  }

  render() {
    var _routeOptions$allowTo2;

    const props = this.props;
    const navigation = this.getRouteNavigationObject();
    const routeOptions = navigation.routeOptions;
    return /*#__PURE__*/React.createElement(NavRouteViewContext.Provider, {
      value: {
        navigation,
        navigatorID: props.navigatorID,
        routeID: props.routeID
      }
    }, /*#__PURE__*/React.createElement(RNINavigatorRouteView, _extends({
      style: styles.navigatorRouteView,
      ref: r => {
        this._nativeRef = r;
      },
      routeID: props.routeID,
      nativeID: NativeIDKeys.NavRouteItem,
      routeKey: props.routeKey,
      routeIndex: props.routeIndex // Route Native Events: Push/Pop Events
      ,
      onRouteWillPop: this._handleOnNavRouteWillPop,
      onRouteDidPop: this._handleOnNavRouteDidPop,
      onRouteWillPush: this._handleOnNavRouteWillPush,
      onRouteDidPush: this._handleOnNavRouteDidPush // Route Native Events: Focus/Blur
      ,
      onRouteWillFocus: this._handleOnRouteWillFocus,
      onRouteDidFocus: this._handleOnRouteDidFocus,
      onRouteWillBlur: this._handleOnRouteWillBlur,
      onRouteDidBlur: this._handleOnRouteDidBlur // Route Native Events: Navbar Item `onPress`
      ,
      onPressNavBarLeftItem: this._handleOnPressNavBarLeftItem,
      onPressNavBarRightItem: this._handleOnPressNavBarRightItem // Route Native Events: Search
      ,
      onUpdateSearchResults: this._handleOnUpdateSearchResults,
      onSearchBarCancelButtonClicked: this._handleOnSearchBarCancelButtonClicked,
      onSearchBarSearchButtonClicked: this._handleOnSearchBarSearchButtonClicked // pass down navbar item config, back button item config, etc.

    }, routeOptions, {
      allowTouchEventsToPassThroughNavigationBar: (_routeOptions$allowTo2 = routeOptions.allowTouchEventsToPassThroughNavigationBar) !== null && _routeOptions$allowTo2 !== void 0 ? _routeOptions$allowTo2 : false
    }), this._renderRouteContents(navigation), /*#__PURE__*/React.createElement(RouteComponentsWrapper, {
      ref: r => {
        this._routeComponentsWrapperRef = r;
      },
      navigation: navigation,
      getPortalRef: this.getPortalRef // render nav bar items
      ,
      renderNavBarLeftItem: props.renderNavBarLeftItem,
      renderNavBarRightItem: props.renderNavBarRightItem,
      renderNavBarTitleItem: props.renderNavBarTitleItem
    })));
  }

}

_defineProperty(NavigatorRouteView, "contextType", NavigatorUIConstantsContext);

;

class RouteViewUtils {
  static isRouteReady(routeStatus) {
    return routeStatus !== RouteStatus.INIT && routeStatus !== RouteStatus.UNMOUNTED;
  }

  static isRouteFocusingOrFocused(routeStatus) {
    return routeStatus === RouteStatus.ROUTE_PUSHING || routeStatus === RouteStatus.ROUTE_PUSHED || routeStatus === RouteStatus.ROUTE_FOCUSING || routeStatus === RouteStatus.ROUTE_FOCUSED;
  }

}

;
const styles = StyleSheet.create({
  routeItem: {
    position: 'absolute',
    width: 0,
    height: 0
  },
  navigatorRouteView: { ...StyleSheet.absoluteFillObject,
    // don't show
    opacity: 0
  },
  routeContentContainer: { ...StyleSheet.absoluteFillObject,
    overflow: 'hidden'
  }
});
//# sourceMappingURL=NavigatorRouteView.js.map