function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

import { CompareUtilities } from './CompareUtilities';
// Note: These functions are used to compare objects and decide whether or not to
// to trigger an update or re-render, so it has to be fast.
// Of course, we can use `deepCompare` and `JSON.stringify` to compare two objects,
// but using `&&` ("short-circuit" eval.) will return early once a comparison is false,
// which is a lot faster than serializing an entire object into a string and then 
// comparing them (it also doesn't preserve order, meaning that the comparisons for
// deeply nested objects might be wrong sometimes).
// This is a ugly brute force approach, and there might be other ways to do this, but
// it'll do for now.
export class CompareImageConfig {
  static compare(oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    if (oldItem.type !== newItem.type) return false;

    switch (oldItem.type) {
      case 'IMAGE_ASSET':
      case 'IMAGE_SYSTEM':
        return oldItem.imageValue !== newItem.imageValue;

      case 'IMAGE_REQUIRE':
      case 'IMAGE_RECT':
      case 'IMAGE_GRADIENT':
        return CompareUtilities.shallowCompareObject(oldItem.imageValue, newItem.imageValue);
    }

    ;
    return true;
  }

}
;
export class CompareNavBarItemConfig {
  static compareTitlePositionAdjustment(oldItem, newItem) {
    if (!CompareUtilities.shallowCompareObject(oldItem, newItem)) {
      return false;
    }

    ;
    let key;

    for (key in oldItem) {
      // compare each Offset object
      if (!CompareUtilities.shallowCompareObject(oldItem[key], newItem[key])) {
        return false;
      }

      ;
    }

    ;
    return true;
  }

  static compareBackgroundImage(oldItem, newItem) {
    if (!CompareUtilities.shallowCompareObject(oldItem, newItem)) {
      return false;
    }

    ;
    let key;

    for (key in oldItem) {
      // compare each background image
      if (!CompareUtilities.shallowCompareObject(oldItem[key], newItem[key])) {
        return false;
      }

      ;
    }

    ;
    return true;
  }

  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareNavBarItemConfig.propertyMap, oldItem, newItem) && // @ts-ignore
    // compare `SupportedImageTypes`
    oldItem.imageValue === newItem.imageValue && // @ts-ignore
    // compare `NavBarItemConfigBase`
    oldItem.title === newItem.title && // @ts-ignore 
    oldItem.systemItem === newItem.systemItem;
  }

}

_defineProperty(CompareNavBarItemConfig, "propertyMap", {
  // shallow compare 
  key: {
    mode: 'shallow'
  },
  type: {
    mode: 'shallow'
  },
  width: {
    mode: 'shallow'
  },
  barButtonItemStyle: {
    mode: 'shallow'
  },
  // shallow compare object
  tintColor: {
    mode: 'shallowObject'
  },
  possibleTitles: {
    mode: 'shallowObject'
  },
  // custom compare
  titlePositionAdjustment: {
    mode: 'custom',
    customCompare: CompareNavBarItemConfig.compareTitlePositionAdjustment
  },
  backgroundImage: {
    mode: 'custom',
    customCompare: CompareNavBarItemConfig.compareBackgroundImage
  }
});

;
export class CompareNavBarButtonBackItemConfig {
  static compare(oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    return (// @ts-ignore
      // compare `SupportedImageTypes`
      oldItem.imageValue === newItem.imageValue && // @ts-ignore
      // compare `NavBarItemConfigBase`
      oldItem.title === newItem.title && // @ts-ignore 
      oldItem.systemItem === newItem.systemItem && CompareUtilities.compareObject(CompareNavBarButtonBackItemConfig.propertyMap, oldItem, newItem)
    );
  }

}

_defineProperty(CompareNavBarButtonBackItemConfig, "propertyMap", { ...CompareNavBarItemConfig.propertyMap
});

;
export class CompareNavBarItemsConfig {
  static compare(oldItem, newItem) {
    if (!CompareUtilities.compareArraySimple(oldItem, newItem)) return false;

    for (let i = 0; (_ref = i < (oldItem === null || oldItem === void 0 ? void 0 : oldItem.length)) !== null && _ref !== void 0 ? _ref : 0; i++) {
      var _ref;

      const oldNavBarItem = oldItem[i];
      const newBavBarItem = newItem[i];

      if (CompareUtilities.isBothNull(oldNavBarItem, newBavBarItem)) {
        return true;
      } else if (!CompareUtilities.compareItemsNull(oldNavBarItem, newBavBarItem)) {
        return false;
      } else if (!CompareNavBarItemConfig.compare(oldNavBarItem, newBavBarItem)) {
        return false;
      }

      ;
    }

    ;
    return true;
  }

}
;
export class CompareRouteTransitionPushConfig {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareRouteTransitionPushConfig.propertyMap, oldItem, newItem);
  }

}

_defineProperty(CompareRouteTransitionPushConfig, "propertyMap", {
  // shallow compare
  type: {
    mode: 'shallow'
  },
  duration: {
    mode: 'shallow'
  }
});

;
export class CompareRouteTransitionPopConfig {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareRouteTransitionPopConfig.propertyMap, oldItem, newItem);
  }

}

_defineProperty(CompareRouteTransitionPopConfig, "propertyMap", {
  // shallow compare
  type: {
    mode: 'shallow'
  },
  duration: {
    mode: 'shallow'
  }
});

;
export class CompareNavBarAppearance {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareNavBarAppearance.propertyMap, oldItem, newItem);
    ;
  }

}

_defineProperty(CompareNavBarAppearance, "propertyMap", {
  // shallow compare
  baseConfig: {
    mode: 'shallow'
  },
  backgroundEffect: {
    mode: 'shallow'
  },
  backgroundColor: {
    mode: 'shallow'
  },
  backgroundImageContentMode: {
    mode: 'shallow'
  },
  shadowColor: {
    mode: 'shallow'
  },
  titlePositionAdjustment: {
    mode: 'shallow'
  },
  // shallow compare object
  titleTextAttributes: {
    mode: 'shallowObject'
  },
  largeTitleTextAttributes: {
    mode: 'shallowObject'
  },
  backIndicatorImage: {
    mode: 'shallowObject'
  },
  // custom compare
  backgroundImage: {
    mode: 'custom',
    customCompare: CompareImageConfig.compare
  },
  shadowImage: {
    mode: 'custom',
    customCompare: CompareImageConfig.compare
  }
});

;
export class CompareNavBarAppearanceConfig {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareNavBarAppearanceConfig.propertyMap, oldItem, newItem);
  }

}

_defineProperty(CompareNavBarAppearanceConfig, "propertyMap", {
  // shallow compare 
  navBarPreset: {
    mode: 'shallow'
  },
  // custom compare
  standardAppearance: {
    mode: 'custom',
    customCompare: CompareNavBarAppearance.compare
  },
  compactAppearance: {
    mode: 'custom',
    customCompare: CompareNavBarAppearance.compare
  },
  scrollEdgeAppearance: {
    mode: 'custom',
    customCompare: CompareNavBarAppearance.compare
  }
});

;
export class CompareLegacyAppearanceConfig {
  static compareBackgroundImage(oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    return CompareImageConfig.compare(oldItem.default, newItem.default) && CompareImageConfig.compare(oldItem.defaultPrompt, newItem.defaultPrompt) && CompareImageConfig.compare(oldItem.compact, newItem.compact) && CompareImageConfig.compare(oldItem.compactPrompt, newItem.compactPrompt);
  }

  static compareTitleVerticalPositionAdjustment(oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    return CompareUtilities.shallowCompareObject(oldItem.default, newItem.default) && CompareUtilities.shallowCompareObject(oldItem.defaultPrompt, newItem.defaultPrompt) && CompareUtilities.shallowCompareObject(oldItem.compact, newItem.compact) && CompareUtilities.shallowCompareObject(oldItem.compactPrompt, newItem.compactPrompt);
  }

  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareLegacyAppearanceConfig.propertyMap, oldItem, newItem);
  }

}

_defineProperty(CompareLegacyAppearanceConfig, "propertyMap", {
  // shallow compare 
  barStyle: {
    mode: 'shallow'
  },
  navBarPreset: {
    mode: 'shallow'
  },
  // shallow object compare
  tintColor: {
    mode: 'shallowObject'
  },
  barTintColor: {
    mode: 'shallowObject'
  },
  backIndicatorImage: {
    mode: 'shallowObject'
  },
  titleTextAttributes: {
    mode: 'shallowObject'
  },
  largeTitleTextAttributes: {
    mode: 'shallowObject'
  },
  // custom compare
  backgroundImage: {
    mode: 'custom',
    customCompare: CompareLegacyAppearanceConfig.compareBackgroundImage
  },
  shadowImage: {
    mode: 'custom',
    customCompare: CompareImageConfig.compare
  },
  titleVerticalPositionAdjustment: {
    mode: 'custom',
    customCompare: CompareLegacyAppearanceConfig.compareTitleVerticalPositionAdjustment
  }
});

;
export class CompareNavBarAppearanceCombinedConfig {
  static compare(oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    return oldItem.mode === newItem.mode && oldItem.navBarPreset === newItem.navBarPreset && (oldItem.mode === 'legacy' ? CompareLegacyAppearanceConfig.compare(oldItem, newItem) : CompareNavBarAppearanceConfig.compare(oldItem, newItem));
  }

}
;
export class CompareRouteOptions {
  static compare(oldItem, newItem) {
    return CompareUtilities.compareObject(CompareRouteOptions.propertyMap, oldItem, newItem);
  }

}

_defineProperty(CompareRouteOptions, "propertyMap", {
  // shallow compare 
  routeTitle: {
    mode: 'shallow'
  },
  prompt: {
    mode: 'shallow'
  },
  largeTitleDisplayMode: {
    mode: 'shallow'
  },
  hidesBackButton: {
    mode: 'shallow'
  },
  backButtonTitle: {
    mode: 'shallow'
  },
  backButtonDisplayMode: {
    mode: 'shallow'
  },
  leftItemsSupplementBackButton: {
    mode: 'shallow'
  },
  navigationBarVisibility: {
    mode: 'shallow'
  },
  statusBarStyle: {
    mode: 'shallow'
  },
  applyBackButtonConfigToCurrentRoute: {
    mode: 'shallow'
  },
  allowTouchEventsToPassThroughNavigationBar: {
    mode: 'shallow'
  },
  automaticallyAddHorizontalSafeAreaInsets: {
    mode: 'shallow'
  },
  // shallow object
  routeContainerStyle: {
    mode: 'shallowObject'
  },
  searchBarConfig: {
    mode: 'shallowObject'
  },
  // custom compare
  navBarAppearanceOverride: {
    mode: 'custom',
    customCompare: CompareNavBarAppearanceCombinedConfig.compare
  },
  transitionConfigPush: {
    mode: 'custom',
    customCompare: CompareRouteTransitionPushConfig.compare
  },
  transitionConfigPop: {
    mode: 'custom',
    customCompare: CompareRouteTransitionPopConfig.compare
  },
  navBarButtonLeftItemsConfig: {
    mode: 'custom',
    customCompare: CompareNavBarItemsConfig.compare
  },
  navBarButtonRightItemsConfig: {
    mode: 'custom',
    customCompare: CompareNavBarItemsConfig.compare
  },
  navBarButtonBackItemConfig: {
    mode: 'custom',
    customCompare: CompareNavBarButtonBackItemConfig.compare
  }
});

;
//# sourceMappingURL=CompareRouteOptions.js.map