export class CompareUtilities {
  /** if one value is null, and the other isn't, then they aren't the same. 
   * Returns true if they're the same, false if different.
  */
  static compareItemsNull(itemA, itemB) {
    return itemA == null === (itemB == null);
  }

  static isBothNull(itemA, itemB) {
    return itemA == null && itemB == null;
  }

  static compareArraySimple(itemA, itemB) {
    if (CompareUtilities.isBothNull(itemA, itemB)) return true;
    return CompareUtilities.compareItemsNull(itemA, itemB) && (itemA === null || itemA === void 0 ? void 0 : itemA.length) === (itemB === null || itemB === void 0 ? void 0 : itemB.length);
  }

  static shallowCompareObject(itemA, itemB) {
    if (CompareUtilities.isBothNull(itemA, itemB)) return true;
    if (!CompareUtilities.compareItemsNull(itemA, itemB)) return false;
    let key;

    for (key in itemA) {
      // skip objects
      if (typeof itemA[key] === 'object') {
        continue;
      }

      ;

      if (itemA[key] !== itemB[key]) {
        return false;
      }

      ;
    }

    ;
    return true;
  }

  static compareObject(propertyMap, oldItem, newItem) {
    if (CompareUtilities.isBothNull(oldItem, newItem)) return true;
    if (!CompareUtilities.compareItemsNull(oldItem, newItem)) return false;
    let key;

    for (key in oldItem) {
      const config = propertyMap[key];

      switch (config === null || config === void 0 ? void 0 : config.mode) {
        case 'shallow':
          if (oldItem[key] !== newItem[key]) return false;
          break;

        case 'custom':
          // @ts-ignore
          if (!config.customCompare(oldItem[key], newItem[key])) return false;
          break;

        case 'shallowObject':
          if (!CompareUtilities.shallowCompareObject(oldItem, newItem)) return false;
          break;

        case 'ignore':
          break;
      }

      ;
    }

    ;
    return true;
  }

}
;
//# sourceMappingURL=CompareUtilities.js.map