import { useContext, useEffect, useRef } from 'react';
import { NavRouteViewContext } from '../context/NavRouteViewContext';
/** Route push/pop events */

function useNavRouteEvents(eventName, handler) {
  const {
    navigation
  } = useContext(NavRouteViewContext); // Create a ref that stores handler

  const savedHandler = useRef(null); // Update ref.current value if handler changes.
  // This allows our effect below to always get latest handler ...
  // ... without us needing to pass it in effect deps array ...
  // ... and potentially cause effect to re-run every render.

  useEffect(() => {
    savedHandler.current = handler;
  }, [handler]);
  useEffect(() => {
    const emitterRef = navigation.getRefToNavRouteEmitter(); // create event listener that calls handler function stored in ref

    const eventListener = params => savedHandler.current(params); // subscribe to events


    emitterRef.addListener(eventName, eventListener);
    return () => {
      // remove event listener on cleanup
      emitterRef.removeListener(eventName, eventListener);
    };
  });
}

;
export function useNavRouteLifeCycle(eventName, handler) {
  useNavRouteEvents(eventName, handler);
}
;
export function useNavBarItemEvents(eventName, handler) {
  useNavRouteEvents(eventName, handler);
}
;
//# sourceMappingURL=useNavRouteEvents.js.map