import React, { ReactElement } from 'react';
import type { NavigationObject } from '../types/NavigationObject';
import type { RouteOptions } from '../types/RouteOptions';
import type { RenderNavBarItem, RenderRouteHeader } from '../types/NavTypes';
import type { NavigatorView } from './NavigatorView';
import type { RouteViewPortal } from './RouteViewPortal';
import { RouteComponentsWrapper } from './RouteComponentsWrapper';
import { RNINavigatorRouteViewProps, RouteTransitionPopConfig, RouteTransitionPushConfig } from '../native_components/RNINavigatorRouteView';
import { EventEmitter } from '../functions/EventEmitter';
import { NavigatorUIConstantsContextProps } from '../context/NavigatorUIConstantsContext';
/** Event emitter keys for `NavigatorRouteView` */
export declare enum NavRouteEvents {
    onRouteWillPush = "onRouteWillPush",
    onRouteDidPush = "onRouteDidPush",
    onRouteWillPop = "onRouteWillPop",
    onRouteDidPop = "onRouteDidPop",
    onRouteWillFocus = "onRouteWillFocus",
    onRouteDidFocus = "onRouteDidFocus",
    onRouteWillBlur = "onRouteWillBlur",
    onRouteDidBlur = "onRouteDidBlur",
    onPressNavBarLeftItem = "onPressNavBarLeftItem",
    onPressNavBarRightItem = "onPressNavBarRightItem",
    onUpdateSearchResults = "onUpdateSearchResults",
    onSearchBarCancelButtonClicked = "onSearchBarCancelButtonClicked",
    onSearchBarSearchButtonClicked = "onSearchBarSearchButtonClicked"
}
export interface RouteContentProps {
    navigation?: NavigationObject;
}
declare enum RouteStatus {
    INIT = "INIT",
    ROUTE_PUSHING = "ROUTE_PUSHING",
    ROUTE_PUSHED = "ROUTE_PUSHED",
    ROUTE_POPPING = "ROUTE_POPPING",
    ROUTE_POPPED = "ROUTE_POPPED",
    ROUTE_BLURRING = "ROUTE_BLURRING",
    ROUTE_BLURRED = "ROUTE_BLURRED",
    ROUTE_FOCUSING = "ROUTE_FOCUSING",
    ROUTE_FOCUSED = "ROUTE_FOCUSED",
    UNMOUNTED = "UNMOUNTED"
}
export declare type NavigatorRouteViewProps = Partial<Pick<RNINavigatorRouteViewProps, 'routeID' | 'routeKey' | 'routeIndex'>> & {
    navigatorID: number;
    routeProps: object;
    isRootRoute: boolean;
    currentActiveRouteIndex: number;
    routeOptionsDefault: RouteOptions;
    transitionConfigPushOverride: RouteTransitionPushConfig;
    transitionConfigPopOverride: RouteTransitionPopConfig;
    getRefToNavigator: () => NavigatorView;
    renderRouteContent: () => ReactElement<RouteContentProps>;
    renderNavBarLeftItem?: RenderNavBarItem;
    renderNavBarRightItem?: RenderNavBarItem;
    renderNavBarTitleItem?: RenderNavBarItem;
    renderRouteHeader?: RenderRouteHeader;
};
/** `NavigatorView` comp. state */
declare type NavigatorRouteViewState = {
    updateIndex: number;
    routeOptions: RouteOptions;
    hasRoutePortal: boolean;
};
export declare class NavigatorRouteView extends React.Component<NavigatorRouteViewProps, NavigatorRouteViewState> {
    static contextType: React.Context<Partial<Pick<{
        target: number;
        navigatorID: number;
    } & {
        statusBarHeight: number;
        safeAreaInsets: import("../types/MiscTypes").EdgeInsets;
    }, "statusBarHeight" | "safeAreaInsets" | "navigatorID">>>;
    state: NavigatorRouteViewState;
    context: NavigatorUIConstantsContextProps;
    routeContentRef: React.Component<RouteContentProps>;
    routeStatus: RouteStatus;
    private _emitter;
    private _nativeRef;
    private _navigatorRef;
    private _routeContentRef;
    private _routeViewPortalRef;
    private _routeComponentsWrapperRef;
    constructor(props: NavigatorRouteViewProps);
    shouldComponentUpdate(nextProps: NavigatorRouteViewProps, nextState: NavigatorRouteViewState): boolean;
    componentWillUnmount(): void;
    getRefToNavigator: () => NavigatorView;
    getRouteRef: () => this;
    getEmitterRef: () => EventEmitter<NavRouteEvents>;
    getPortalRef: () => RouteViewPortal;
    getRouteComponentsWrapper: () => RouteComponentsWrapper;
    /** Combines all the route configs into one */
    getRouteOptions: () => RouteOptions;
    getRouteNavigationObject: () => NavigationObject;
    setRouteOptions: (routeOptions: RouteOptions) => Promise<void>;
    setRouteViewPortalRef: (ref: RouteViewPortal) => void;
    setHidesBackButton: (isHidden: boolean, animated: boolean) => Promise<void>;
    getRouteConstants: () => Promise<import("../native_modules/RNINavigatorRouteViewModule").RouteConstantsObject>;
    private _handleGetRefToRoute;
    private _handleGetRefToNavRouteEmitter;
    private _handleOnNavRouteWillPop;
    private _handleOnNavRouteDidPop;
    private _handleOnNavRouteWillPush;
    private _handleOnNavRouteDidPush;
    private _handleOnPressNavBarLeftItem;
    private _handleOnPressNavBarRightItem;
    private _handleOnRouteWillFocus;
    private _handleOnRouteDidFocus;
    private _handleOnRouteWillBlur;
    private _handleOnRouteDidBlur;
    private _handleOnUpdateSearchResults;
    private _handleOnSearchBarCancelButtonClicked;
    private _handleOnSearchBarSearchButtonClicked;
    _renderRouteContents: (navigation: NavigationObject) => JSX.Element;
    render(): JSX.Element;
}
export {};
