import React from 'react';
import { NavRouteEvents } from '../components/NavigatorRouteView';
import type { NavigatorRouteView } from '../components/NavigatorRouteView';
import type { onRoutePopEvent, onRoutePushEvent, onRouteFocusBlurEvent, onPressNavBarItem, onUpdateSearchResults, onSearchBarCancelButtonClicked, onSearchBarSearchButtonClicked } from 'src/native_components/RNINavigatorRouteView';
import type { EventEmitter } from '../functions/EventEmitter';
import type { RouteOptions } from '../types/RouteOptions';
import { NavRouteViewContextProps } from '../context/NavRouteViewContext';
declare type RouteViewEventsProps = {
    routeOptions?: RouteOptions;
    onRouteWillPush?: onRoutePushEvent;
    onRouteDidPush?: onRoutePushEvent;
    onRouteWillPop?: onRoutePopEvent;
    onRouteDidPop?: onRoutePopEvent;
    onRouteWillFocus?: onRouteFocusBlurEvent;
    onRouteDidFocus?: onRouteFocusBlurEvent;
    onRouteWillBlur?: onRouteFocusBlurEvent;
    onRouteDidBlur?: onRouteFocusBlurEvent;
    onPressNavBarLeftItem?: onPressNavBarItem;
    onPressNavBarRightItem?: onPressNavBarItem;
    onUpdateSearchResults?: onUpdateSearchResults;
    onSearchBarCancelButtonClicked?: onSearchBarCancelButtonClicked;
    onSearchBarSearchButtonClicked?: onSearchBarSearchButtonClicked;
};
/**
 * This component is used to listen to route-related events.
 *
 * This component is used inside a route, and allows you to listen for the events
 * that are related to that route.
 *
 * This component is just a "convenient" wrapper component that subscribes to the
 * parent route component's event emitter. The reference to the event emitter is obtained
 * via context.
 */
export declare class RouteViewEvents extends React.Component<RouteViewEventsProps> {
    static contextType: React.Context<Partial<NavRouteViewContextProps>>;
    context: NavRouteViewContextProps;
    routerRef: NavigatorRouteView;
    emitterRef: EventEmitter<NavRouteEvents>;
    constructor(props: RouteViewEventsProps, context: NavRouteViewContextProps);
    componentWillUnmount(): void;
    _handleOnRouteWillPush: onRoutePushEvent;
    _handleOnRouteDidPush: onRoutePushEvent;
    _handleOnRouteWillPop: onRoutePopEvent;
    _handleOnRouteDidPop: onRoutePopEvent;
    _handleOnRouteWillFocus: onRouteFocusBlurEvent;
    _handleOnRouteDidFocus: onRouteFocusBlurEvent;
    _handleOnRouteWillBlur: onRouteFocusBlurEvent;
    _handleOnRouteDidBlur: onRouteFocusBlurEvent;
    _handleOnPressNavBarLeftItem: onPressNavBarItem;
    _handleOnPressNavBarRightItem: onPressNavBarItem;
    _handleOnUpdateSearchResults: onUpdateSearchResults;
    _handleOnSearchBarCancelButtonClicked: onSearchBarCancelButtonClicked;
    _handleOnSearchBarSearchButtonClicked: onSearchBarSearchButtonClicked;
    render(): any;
}
export {};
