import { ViewStyle } from 'react-native';
import type { NavBarBackItemConfig, NavBarItemsConfig } from '../types/NavBarItemConfig';
import type { NavBarAppearanceCombinedConfig } from '../types/NavBarAppearanceConfig';
import type { RouteSearchControllerConfig } from '../types/RouteSearchControllerConfig';
declare type RouteTransitionPushTypes = "DefaultPush" | "FadePush" | "SlideLeftPush" | "SlideUpPush" | "GlideUpPush";
declare type RouteTransitionPopTypes = "DefaultPop" | "FadePop" | "SlideLeftPop" | "SlideUpPop" | "GlideUpPop";
declare type RouteTransitionConfigBase = {
    duration?: number;
};
export declare type RouteTransitionPushConfig = RouteTransitionConfigBase & {
    type: RouteTransitionPushTypes;
};
export declare type RouteTransitionPopConfig = RouteTransitionConfigBase & {
    type: RouteTransitionPopTypes;
};
declare type BaseEventPayload = {
    routeID: number;
    routeKey: string;
    routeIndex: number;
};
export declare type onRoutePushEvent = (event: BaseEventPayload & {
    nativeEvent: BaseEventPayload & {
        isAnimated: boolean;
    };
}) => void;
export declare type onRouteFocusBlurEvent = (event: {
    nativeEvent: BaseEventPayload & {
        isAnimated: boolean;
    };
}) => void;
export declare type onRoutePopEvent = (event: {
    nativeEvent: BaseEventPayload & {
        isUserInitiated: boolean;
    };
}) => void;
export declare type onPressNavBarItem = (event: {
    nativeEvent: BaseEventPayload & {
        key: string;
        type: string;
        index: string;
    };
}) => void;
export declare type onUpdateSearchResults = (event: {
    nativeEvent: BaseEventPayload & {
        text?: string;
        isActive: boolean;
    };
}) => void;
export declare type onSearchBarCancelButtonClicked = (event: {
    nativeEvent: BaseEventPayload;
}) => void;
export declare type onSearchBarSearchButtonClicked = (event: {
    nativeEvent: BaseEventPayload & {
        text?: string;
    };
}) => void;
export declare type BackButtonDisplayMode = 
/** The navigation item attempts to display a specific title, a generic title, or no title for the Back button, depending on the space available. */
'default'
/**The navigation item attempts to display a generic title or no title for the Back button, depending on the space available. */
 | 'generic'
/** The navigation item displays the Back button indicator instead of a title. */
 | 'minimal';
export declare type LargeTitleDisplayMode = 
/** Inherit the display mode from the previous navigation item. */
'automatic'
/** Always display a large title. */
 | 'always'
/** Never display a large title. */
 | 'never';
export declare type NavigationBarVisibilityMode = 'visible' | 'hidden' | 'default';
export declare type StatusBarStyle = "default" | "lightContent" | "darkContent";
export declare type RNINavigatorRouteViewProps = {
    style: ViewStyle;
    routeID: number;
    nativeID: string;
    routeKey: string;
    routeIndex: number;
    statusBarStyle?: StatusBarStyle;
    transitionConfigPush?: RouteTransitionPushConfig;
    transitionConfigPop?: RouteTransitionPopConfig;
    prompt?: String;
    routeTitle?: string;
    largeTitleDisplayMode?: LargeTitleDisplayMode;
    searchBarConfig?: RouteSearchControllerConfig;
    navBarButtonBackItemConfig?: NavBarBackItemConfig;
    navBarButtonLeftItemsConfig?: NavBarItemsConfig;
    navBarButtonRightItemsConfig?: NavBarItemsConfig;
    hidesBackButton?: boolean;
    backButtonTitle?: string;
    backButtonDisplayMode?: BackButtonDisplayMode;
    leftItemsSupplementBackButton?: boolean;
    applyBackButtonConfigToCurrentRoute?: boolean;
    navBarAppearanceOverride?: NavBarAppearanceCombinedConfig;
    navigationBarVisibility?: NavigationBarVisibilityMode;
    allowTouchEventsToPassThroughNavigationBar: boolean;
    onRouteWillPush?: onRoutePushEvent;
    onRouteDidPush?: onRoutePushEvent;
    onRouteWillPop?: onRoutePopEvent;
    onRouteDidPop?: onRoutePopEvent;
    onRouteWillFocus?: onRouteFocusBlurEvent;
    onRouteDidFocus?: onRouteFocusBlurEvent;
    onRouteWillBlur?: onRouteFocusBlurEvent;
    onRouteDidBlur?: onRouteFocusBlurEvent;
    onPressNavBarLeftItem?: onPressNavBarItem;
    onPressNavBarRightItem?: onPressNavBarItem;
    /** Request to update the search results based on the search text. */
    onUpdateSearchResults?: onUpdateSearchResults;
    /** The cancel button was tapped. */
    onSearchBarCancelButtonClicked?: onSearchBarCancelButtonClicked;
    /** The search button (usually in the keyboard) was tapped. */
    onSearchBarSearchButtonClicked?: onSearchBarSearchButtonClicked;
};
export declare const RNINavigatorRouteView: import("react-native").HostComponent<RNINavigatorRouteViewProps>;
export {};
