import { ViewStyle } from 'react-native';
import type { EdgeInsets } from 'src/types/MiscTypes';
import type { NavBarAppearanceCombinedConfig } from 'src/types/NavBarAppearanceConfig';
declare type EventBasePayload = {
    target: number;
    navigatorID: number;
};
export declare type OnNavRouteViewAddedPayload = {
    nativeEvent: EventBasePayload & {
        routeID: number;
        routeKey: string;
        routeIndex: number;
        isNativeRoute: boolean;
    };
};
export declare type OnNavRouteWillPopPayload = {
    nativeEvent: EventBasePayload & {
        routeKey: string;
        routeIndex: number;
        isUserInitiated: boolean;
    };
};
export declare type OnNavRouteDidPopPayload = {
    nativeEvent: EventBasePayload & {
        routeKey: string;
        routeIndex: number;
        isUserInitiated: boolean;
    };
};
export declare type OnSetNativeRouteDataPayload = {
    nativeEvent: EventBasePayload;
};
export declare type OnNativeCommandRequestPayload = {
    nativeEvent: EventBasePayload & {
        commandData: {
            commandKey: 'pushViewController';
            routeID: number;
            routeKey: string;
            isAnimated: boolean;
        } | {
            commandKey: 'push';
            routeKey: string;
            routeProps?: object;
            isAnimated: boolean;
        } | {
            commandKey: 'pop';
            isAnimated: boolean;
        };
    };
};
export declare type OnCustomCommandFromNativePayload = {
    nativeEvent: EventBasePayload & {
        commandKey: string;
        commandData: object;
    };
};
export declare type OnUIConstantsDidChangePayload = {
    nativeEvent: EventBasePayload & {
        statusBarHeight: number;
        safeAreaInsets: EdgeInsets;
    };
};
export declare type NativeRouteMap = {
    [key: string]: {
        routeKey: string;
        routeIndex: number;
        routeProps?: object;
    };
};
/** `RNINavigatorView` native comp. props */
export declare type RNINavigatorViewProps = {
    style: ViewStyle | Array<ViewStyle>;
    nativeID?: string;
    navigatorID: number;
    nativeRoutes: NativeRouteMap;
    initialRouteKeys: Array<string>;
    isInteractivePopGestureEnabled: boolean;
    shouldSwizzleRootViewController: boolean;
    navBarPrefersLargeTitles: boolean;
    isNavBarTranslucent: boolean;
    navBarAppearance: NavBarAppearanceCombinedConfig;
    onNavRouteViewAdded?: (event: OnNavRouteViewAddedPayload) => void;
    onSetNativeRoutes?: (event: OnSetNativeRouteDataPayload) => void;
    onNavRouteWillPop?: (event: OnNavRouteWillPopPayload) => void;
    onNavRouteDidPop?: (event: OnNavRouteDidPopPayload) => void;
    onNativeCommandRequest?: (event: OnNativeCommandRequestPayload) => void;
    onCustomCommandFromNative?: (event: OnCustomCommandFromNativePayload) => void;
    onUIConstantsDidChange?: (event: OnUIConstantsDidChangePayload) => void;
};
export declare type RNINavigatorViewConstantsObject = {
    navigationBarHeight: number;
};
export declare const RNINavigatorView: import("react-native").HostComponent<RNINavigatorViewProps>;
export declare const RNINavigatorViewConstants: RNINavigatorViewConstantsObject;
export {};
