import type { RNINavigatorRouteViewProps } from 'src/native_components/RNINavigatorRouteView';
import type { EdgeInsets, Rect } from 'src/types/MiscTypes';
export declare type NativePushPopOptions = {
    isAnimated?: boolean;
};
declare type RouteData = {
    type: 'viewController';
} | ({
    type: 'reactRoute' | 'nativeRoute';
} & Pick<RNINavigatorRouteViewProps, 'routeID' | 'routeKey' | 'routeIndex'>);
export declare type NavigatorConstantsObject = {
    navigatorID: number;
    navBarHeight: number;
    statusBarHeight: number;
    safeAreaInsets: EdgeInsets;
    bounds: Rect;
    isPresenting: boolean;
    activeRoutes: Array<RouteData>;
    topViewController?: RouteData;
    visibleViewController?: RouteData;
};
interface RNINavigatorViewModule {
    push(node: number, routeID: number, options: NativePushPopOptions): Promise<void>;
    pop(node: number, options: NativePushPopOptions): Promise<{
        routeKey: string;
        routeIndex: number;
    }>;
    setNavigationBarHidden(node: number, isHidden: boolean, animated: boolean): Promise<void>;
    popToRoot(node: number, options: NativePushPopOptions): Promise<void>;
    removeRoute(node: number, routeID: number, routeIndex: number, animated: boolean): Promise<void>;
    removeRoutes(node: number, itemsToRemove: Array<{
        routeID: number;
        routeIndex: number;
    }>, animated: boolean): Promise<void>;
    replaceRoute(node: number, prevRouteIndex: number, prevRouteID: number, nextRouteID: number, animated: boolean): Promise<void>;
    insertRoute(node: number, nextRouteID: number, atIndex: number, animated: boolean): Promise<void>;
    setRoutes(node: number, nextRouteIDs: Array<number>, animated: boolean): Promise<void>;
    getNativeRouteKeys(callback: (keys: [string]) => void): void;
    sendCustomCommandToNative(node: number, commandKey: string, commandData: object | null): Promise<object | null>;
    getNavigatorConstants(node: number): Promise<NavigatorConstantsObject>;
}
export declare const RNINavigatorViewModule: RNINavigatorViewModule;
export {};
