import { requireNativeComponent, ViewStyle } from 'react-native';

import type { NavBarBackItemConfig, NavBarItemsConfig } from '../types/NavBarItemConfig';
import type { NavBarAppearanceCombinedConfig } from '../types/NavBarAppearanceConfig';
import type { RouteSearchControllerConfig } from '../types/RouteSearchControllerConfig';


// TODO: Rename types - Capitalize, e.g. rename `onXXX` to `OnXXX`.
// TODO: Cleanup types - Replace `,` with `;` in object declaration.

type RouteTransitionPushTypes = 
  "DefaultPush" | "FadePush" | "SlideLeftPush" | "SlideUpPush" | "GlideUpPush";

type RouteTransitionPopTypes = 
  "DefaultPop" | "FadePop" | "SlideLeftPop" | "SlideUpPop" | "GlideUpPop";

type RouteTransitionConfigBase = {
  duration?: number;
};

export type RouteTransitionPushConfig = RouteTransitionConfigBase & {
  type: RouteTransitionPushTypes;
};

export type RouteTransitionPopConfig = RouteTransitionConfigBase & {
  type: RouteTransitionPopTypes;
};

type BaseEventPayload = {
  routeID: number;
  routeKey: string;
  routeIndex: number;
};

export type onRoutePushEvent = (event: BaseEventPayload & {
  nativeEvent: BaseEventPayload & { 
    isAnimated: boolean;
  };
}) => void;

export type onRouteFocusBlurEvent = (event: {
  nativeEvent: BaseEventPayload & { 
    isAnimated: boolean;
  };
}) => void;

export type onRoutePopEvent = (event: {
  nativeEvent: BaseEventPayload & {
    isUserInitiated: boolean;
  };
}) => void;

export type onPressNavBarItem = (event: {
  nativeEvent: BaseEventPayload & { 
    key  : string;
    type : string;
    index: string;
  };
}) => void;

export type onUpdateSearchResults = (event: {
  nativeEvent: BaseEventPayload & { 
    text?: string;
    isActive: boolean;
  };
}) => void;

export type onSearchBarCancelButtonClicked = (event: {
  nativeEvent: BaseEventPayload;
}) => void;

export type onSearchBarSearchButtonClicked = (event: {
  nativeEvent: BaseEventPayload & { 
    text?: string;
  };
}) => void;


export type BackButtonDisplayMode =
  /** The navigation item attempts to display a specific title, a generic title, or no title for the Back button, depending on the space available. */
  | 'default'
  /**The navigation item attempts to display a generic title or no title for the Back button, depending on the space available. */
  | 'generic'
  /** The navigation item displays the Back button indicator instead of a title. */
  | 'minimal';

export type LargeTitleDisplayMode =
  /** Inherit the display mode from the previous navigation item. */
  | 'automatic'
  /** Always display a large title. */
  | 'always'
  /** Never display a large title. */
  | 'never';

export type NavigationBarVisibilityMode = 
  'visible' | 'hidden' | 'default';

export type StatusBarStyle = 
  "default" | "lightContent" | "darkContent";


export type RNINavigatorRouteViewProps = {
  style: ViewStyle;

  routeID: number;
  nativeID: string;
  routeKey: string;
  routeIndex: number;

  statusBarStyle?: StatusBarStyle;

  // Transition Config
  transitionConfigPush?: RouteTransitionPushConfig;
  transitionConfigPop ?: RouteTransitionPopConfig;

  // Navbar Config
  prompt?: String;
  routeTitle?: string;
  largeTitleDisplayMode?: LargeTitleDisplayMode;

  searchBarConfig?: RouteSearchControllerConfig;

  // Navbar Item Config
  navBarButtonBackItemConfig  ?: NavBarBackItemConfig;
  navBarButtonLeftItemsConfig ?: NavBarItemsConfig;
  navBarButtonRightItemsConfig?: NavBarItemsConfig;

  // Navbar back button item config
  hidesBackButton?: boolean;
  backButtonTitle?: string;
  backButtonDisplayMode?: BackButtonDisplayMode;
  leftItemsSupplementBackButton?: boolean;
  applyBackButtonConfigToCurrentRoute?: boolean;
  
  // `NavigationConfigOverride`-related
  navBarAppearanceOverride?: NavBarAppearanceCombinedConfig;
  navigationBarVisibility?: NavigationBarVisibilityMode;
  allowTouchEventsToPassThroughNavigationBar: boolean;

  // Native Events: Push
  onRouteWillPush?: onRoutePushEvent;
  onRouteDidPush ?: onRoutePushEvent;

  // Native Events: Pop
  onRouteWillPop?: onRoutePopEvent;
  onRouteDidPop ?: onRoutePopEvent;

  // Native Events: Focus
  onRouteWillFocus?: onRouteFocusBlurEvent;
  onRouteDidFocus ?: onRouteFocusBlurEvent;

  // Native Events: Blur
  onRouteWillBlur?: onRouteFocusBlurEvent;
  onRouteDidBlur ?: onRouteFocusBlurEvent;

  // Native Events: Navbar Item `onPress`
  onPressNavBarLeftItem ?: onPressNavBarItem;
  onPressNavBarRightItem?: onPressNavBarItem;

  // Native Events: Search
  /** Request to update the search results based on the search text. */
  onUpdateSearchResults?: onUpdateSearchResults;

  /** The cancel button was tapped. */
  onSearchBarCancelButtonClicked?: onSearchBarCancelButtonClicked;
  
  /** The search button (usually in the keyboard) was tapped. */
  onSearchBarSearchButtonClicked?: onSearchBarSearchButtonClicked;
};

export const RNINavigatorRouteView = 
  requireNativeComponent<RNINavigatorRouteViewProps>('RNINavigatorRouteView');